@@ -1,33 +1,35 @@
-from urllib import urlopen, urlencode
+from __future__ import absolute_import
 
-from gobject import SIGNAL_RUN_FIRST, TYPE_NONE
+from gi.repository import GObject
 
-from pychess.System.ThreadPool import pool
+from threading import Thread
+
+from pychess.compat import urlopen, urlencode
+from pychess.System import fident
 from pychess.System.Log import log
 from pychess.Utils.Offer import Offer
 from pychess.Utils.const import ARTIFICIAL, CHAT_ACTION
 
-
-from Player import Player
+from .Player import Player
 
 class Engine (Player):
     
     __type__ = ARTIFICIAL
     
-    ''' The first argument is the pv list of moves. The second is a score
+    ''' Argument is a vector of analysis lines.
+        The first element is the pv list of moves. The second is a score
         relative to the engine. If no score is known, the value can be None,
         but not 0, which is a draw. '''
-    __gsignals__ = {
-        'analyze': (SIGNAL_RUN_FIRST, TYPE_NONE, (object,object))
+    __gsignals__ = {       
+        'analyze': (GObject.SignalFlags.RUN_FIRST, None, (object,))
     }
     
-    def __init__(self):
+    def __init__(self, md5=None):
         Player.__init__(self)
+        self.md5 = md5
         
         self.currentAnalysis = []
-        def on_analysis(self_, analysis, score):
-            if score != None:
-                self.currentScore = score
+        def on_analysis(self_, analysis):
             self.currentAnalysis = analysis
         self.connect('analyze', on_analysis)
     
@@ -83,6 +85,18 @@ class Engine (Player):
     def canAnalyze (self):
         raise NotImplementedError
     
+    def maxAnalysisLines (self):
+        raise NotImplementedError
+    
+    def requestMultiPV (self, setting):
+        """Set the number of analysis lines the engine will give, if possible.
+            
+        If setting is too high, the engine's maximum will be used.
+        The setting will last until the next call to requestMultiPV.
+        Return value: the setting used.
+        """
+        raise NotImplementedError
+    
     def getAnalysis (self):
         """ Returns a list of moves, or None if there haven't yet been made an
             analysis """
@@ -96,13 +110,15 @@ class Engine (Player):
         def answer (message):
             try:
                 data = urlopen("http://www.pandorabots.com/pandora/talk?botid=8d034368fe360895",
-                               urlencode({"message":message, "botcust2":"x"})).read()
-            except IOError, e:
-                log.warn("Couldn't answer message from online bot: '%s'\n" % e,
-                         self.defname)
+                               urlencode({"message":message, "botcust2":"x"}).encode("utf-8")).read().decode('utf-8')
+            except IOError as e:
+                log.warning("Couldn't answer message from online bot: '%s'" % e,
+                         extra={"task":self.defname})
                 return
             ss = "<b>DMPGirl:</b>"
             es = "<br>"
             answer = data[data.find(ss)+len(ss) : data.find(es,data.find(ss))]
             self.emit("offer", Offer(CHAT_ACTION, answer))
-        pool.start(answer, message)
+        t = Thread(target=answer, name=fident(answer), args=(message,))
+        t.daemon = True
+        t.start()
