@@ -1,4 +1,4 @@
-from gobject import GObject, SIGNAL_RUN_FIRST, TYPE_NONE
+from gi.repository import GObject
 
 class PlayerIsDead (Exception):
     """ Used instead of returning a move,
@@ -6,31 +6,46 @@ class PlayerIsDead (Exception):
     pass
 
 class TurnInterrupt (Exception):
-    """ Used instead of returning a move, when a players turn is interupted.
+    """ Used instead of returning a move, when a players turn is interrupted.
         Currently this will only happen when undoMoves changes the current
         player """
     pass
 
-class Player (GObject):
+class InvalidMove (Exception):
+    """ Used instead of returning a move,
+        when an engine plays an invalid move """
+    pass
+
+
+class Player (GObject.GObject):
     
     __gsignals__ = {
-        "offer": (SIGNAL_RUN_FIRST, TYPE_NONE, (object,)),
-        "withdraw": (SIGNAL_RUN_FIRST, TYPE_NONE, (object,)),
-        "decline": (SIGNAL_RUN_FIRST, TYPE_NONE, (object,)),
-        "accept": (SIGNAL_RUN_FIRST, TYPE_NONE, (object,))
+        "offer": (GObject.SignalFlags.RUN_FIRST, None, (object,)),
+        "withdraw": (GObject.SignalFlags.RUN_FIRST, None, (object,)),
+        "decline": (GObject.SignalFlags.RUN_FIRST, None, (object,)),
+        "accept": (GObject.SignalFlags.RUN_FIRST, None, (object,)),
+        "name_changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+
     }
     
     def __init__ (self):
-        GObject.__init__(self)
+        GObject.GObject.__init__(self)
         self.name = ""
+        self.ichandle = None
+        self.icrating = None
     
     def setName (self, name):
         """ __repr__ should return this name """
         self.name = name
+        self.emit("name_changed")
     
     def __repr__ (self):
         return self.name
     
+    @property
+    def time (self):
+        pass  # Optional
+    
     #===========================================================================
     #    Starting the game
     #===========================================================================
@@ -102,13 +117,13 @@ class Player (GObject):
     
     def playerUndoMoves (self, moves, gamemodel):
         """ Some players undo different depending on whether they are players or
-            spectactors. This is a convenient way to handle that. """  
+            spectators. This is a convenient way to handle that. """  
         #Optional
         return self.undoMoves (moves, gamemodel)
     
     def spectatorUndoMoves (self, moves, gamemodel):
         """ Some players undo different depending on whether they are players or
-            spectactors. This is a convenient way to handle that. """  
+            spectators. This is a convenient way to handle that. """  
         #Optional
         return self.undoMoves (moves, gamemodel)
     
