@@ -1,4 +1,4 @@
-from gobject import SIGNAL_RUN_FIRST
+from gi.repository import GObject
 from threading import Condition
 
 from pychess.System.Log import log
@@ -9,16 +9,16 @@ from pychess.Utils.repr import reprColor
 class ProtocolEngine (Engine):
     
     __gsignals__ = {
-        "readyForOptions": (SIGNAL_RUN_FIRST, None, ()),
-        "readyForMoves": (SIGNAL_RUN_FIRST, None, ())
+       "readyForOptions": (GObject.SignalFlags.RUN_FIRST, None, ()),
+       "readyForMoves": (GObject.SignalFlags.RUN_FIRST, None, ())
     }
     
     #===========================================================================
     #    Setting engine options
     #===========================================================================
     
-    def __init__ (self, subprocess, color, protover):
-        Engine.__init__(self)
+    def __init__ (self, subprocess, color, protover, md5):
+        Engine.__init__(self, md5)
         
         self.engine = subprocess
         self.defname = subprocess.defname
@@ -31,6 +31,6 @@ class ProtocolEngine (Engine):
         self.connected = True
         self.mode = NORMAL
         
-        log.debug(reprColor[color]+"\n", self.defname)
+        log.debug(reprColor[color], extra={"task":self.defname})
         
         self.movecon = Condition()
