@@ -1,31 +1,35 @@
-from __future__ import with_statement
+from __future__ import absolute_import
+from __future__ import print_function
+
 import collections
 from copy import copy
-import Queue
-from threading import RLock
+from threading import RLock, Thread
 
+from pychess.compat import Queue
 from pychess.Utils.Move import *
 from pychess.Utils.Board import Board
 from pychess.Utils.Cord import Cord
 from pychess.Utils.Offer import Offer
-from pychess.Utils.logic import validate, getMoveKillingKing, getStatus
+from pychess.Utils.Move import toAN, parseAN, listToMoves
+from pychess.Utils.logic import validate, getMoveKillingKing, getStatus, legalMoveCount
 from pychess.Utils.const import *
 from pychess.Utils.lutils.ldata import MATE_VALUE
+from pychess.Utils.lutils.lmove import ParsingError
+from pychess.System import conf, fident
 from pychess.System.Log import log
 from pychess.System.SubProcess import TimeOutError, SubProcessError
-from pychess.System.ThreadPool import pool
-from pychess.Variants.fischerandom import FischerRandomChess
+from pychess.Variants.fischerandom import FischerandomBoard
 
-from ProtocolEngine import ProtocolEngine
-from Player import Player, PlayerIsDead, TurnInterrupt
+from .ProtocolEngine import ProtocolEngine
+from pychess.Players.Player import Player, PlayerIsDead, TurnInterrupt, InvalidMove
 
 TYPEDIC = {"check":lambda x:x=="true", "spin":int}
-OPTKEYS = ("type", "min", "max", "default", "var")
+OPTKEYS = ("name", "type", "min", "max", "default", "var")
 
 class UCIEngine (ProtocolEngine):
     
-    def __init__ (self, subprocess, color, protover):
-        ProtocolEngine.__init__(self, subprocess, color, protover)
+    def __init__ (self, subprocess, color, protover, md5):
+        ProtocolEngine.__init__(self, subprocess, color, protover, md5)
         
         self.ids = {}
         self.options = {}
@@ -39,19 +43,26 @@ class UCIEngine (ProtocolEngine):
         self.moveLock = RLock()
         # none of the following variables should be changed or used in a
         # condition statement without holding the above self.moveLock
+        self.ponderOn = False
         self.pondermove = None
         self.ignoreNext = False
         self.waitingForMove = False
         self.needBestmove = False
         self.readyForStop = False   # keeps track of whether we already sent a 'stop' command
+        self.multipvSetting  = conf.get("multipv", 1)    # MultiPV option sent to the engine
+        self.multipvExpected = 1    # Number of PVs expected (limited by number of legal moves)
         self.commands = collections.deque()
         
-        self.board = None
-        self.uciok = False
+        self.gameBoard = Board(setup=True) # board at the end of all moves played
+        self.board = Board(setup=True)     # board to send the engine
+        self.uciPosition = "startpos"
+        self.uciPositionListsMoves = False
+        self.analysis = [ None ]
         
-        self.returnQueue = Queue.Queue()
-        self.engine.connect("line", self.parseLines)
+        self.returnQueue = Queue()
+        self.engine.connect("line", self.parseLine)
         self.engine.connect("died", self.__die)
+        self.invalid_move = None
         
         self.connect("readyForOptions", self.__onReadyForOptions_before)
         self.connect_after("readyForOptions", self.__onReadyForOptions)
@@ -65,11 +76,14 @@ class UCIEngine (ProtocolEngine):
     #===========================================================================
     
     def prestart (self):
-        print >> self.engine, "uci"
+        print("uci", file=self.engine)
     
     def start (self):
         if self.mode in (ANALYZING, INVERSE_ANALYZING):
-            pool.start(self.__startBlocking)
+            t = Thread(target=self.__startBlocking,
+                       name=fident(self.__startBlocking))
+            t.daemon = True
+            t.start()
         else:
             self.__startBlocking()
     
@@ -89,13 +103,15 @@ class UCIEngine (ProtocolEngine):
             if self.hasOption("Ponder"):
                 self.setOption('Ponder', False)
         
-        for option, value in self.optionsToBeSent.iteritems():
-            if self.options[option]["default"] != value:
-                self.options[option]["default"] = value
-                if type(value) == bool: value = str(value).lower()
-                print >> self.engine, "setoption name", option, "value", str(value)
+            if self.hasOption("MultiPV") and self.multipvSetting > 1:
+                self.setOption('MultiPV', self.multipvSetting)
+            
+        for option, value in self.optionsToBeSent.items():
+            if isinstance(value, bool):
+                value = str(value).lower()
+            print("setoption name %s value %s" % (option, str(value)), file=self.engine)
         
-        print >> self.engine, "isready"
+        print("isready", file=self.engine)
     
     def __onReadyForMoves (self, self_):
         self.returnQueue.put("ready")
@@ -105,9 +121,7 @@ class UCIEngine (ProtocolEngine):
         # If we are an analyzer, this signal was already called in a different
         # thread, so we can safely block it.
         if self.mode in (ANALYZING, INVERSE_ANALYZING):
-            if not self.board:
-                self.board = Board(setup=True)
-            self.putMove(self.board, None, None)
+            self._searchNow()
     
     #===========================================================================
     #    Ending the game
@@ -115,6 +129,8 @@ class UCIEngine (ProtocolEngine):
     
     def end (self, status, reason):
         # UCI doens't care about reason, so we just kill
+        if reason == WON_ADJUDICATION:
+            self.returnQueue.put("invalid")
         self.kill(reason)
     
     def kill (self, reason):
@@ -126,46 +142,85 @@ class UCIEngine (ProtocolEngine):
             self.connected = False
             try:
                 try:
-                    print >> self.engine, "stop"
-                    print >> self.engine, "quit"
+                    print("stop", file=self.engine)
+                    print("quit", file=self.engine)
                     self.returnQueue.put("del")
                     return self.engine.gentleKill()
                 
-                except OSError, e:
+                except OSError as e:
                     # No need to raise on a hang up error, as the engine is dead
                     # anyways
                     if e.errno == 32:
-                        log.warn("Hung up Error", self.defname)
+                        log.warning("Hung up Error", extra={"task":self.defname})
                         return e.errno
                     else: raise
             
             finally:
                 # Clear the analyzed data, if any
-                self.emit("analyze", [], None)
+                self.emit("analyze", [])
     
     #===========================================================================
     #    Send the player move updates
     #===========================================================================
     
-    def putMove (self, board1, move, board2):
-        log.debug("putMove: board1=%s move=%s board2=%s self.board=%s\n" % \
-            (board1, move, board2, self.board), self.defname)
-        if not self.readyMoves: return
-        
-        self.board = board1
-        
+    def _moveToUCI (self, board, move):
+        cn = CASTLE_KK
+        if board.variant == FISCHERRANDOMCHESS:
+            cn = CASTLE_KR
+        return toAN(board, move, short=True, castleNotation=cn)
+    
+    def _recordMove (self, board1, move, board2):
+        if self.gameBoard == board1:
+            return
+        if not board2:
+            if board1.variant == NORMALCHESS and board1.asFen() == FEN_START:
+                self.uciPosition = "startpos"
+            else:
+                self.uciPosition = "fen " + board1.asFen()
+            self.uciPositionListsMoves = False
+        if move:
+            if not self.uciPositionListsMoves:
+                self.uciPosition += " moves"
+                self.uciPositionListsMoves = True
+            self.uciPosition += " " + self._moveToUCI(board2, move)
+
+
+        self.board = self.gameBoard = board1
         if self.mode == INVERSE_ANALYZING:
-            self.board = self.board.switchColor()
+            self.board = self.gameBoard.switchColor()
+
+    def _recordMoveList (self, model, ply=None):
+        self._recordMove(model.boards[0], None, None)
+        if ply is None:
+            ply = model.ply
+        for board1, move, board2 in zip(model.boards[1:ply+1], model.moves, model.boards[0:ply]):
+            self._recordMove(board1, move, board2)
+
+    
+    def setBoard (self, board):
+        log.debug("setBoardAtPly: board=%s" % board, extra={"task":self.defname})
+        self._recordMove(board, None, None)
         
+        if not self.readyMoves:
+            return
+        self._searchNow()
+
+    def putMove (self, board1, move, board2):
+        log.debug("putMove: board1=%s move=%s board2=%s self.board=%s" % \
+            (board1, move, board2, self.board), extra={"task":self.defname})
+        self._recordMove(board1, move, board2)
+        
+        if not self.readyMoves:
+            return
         self._searchNow()
     
     def makeMove (self, board1, move, board2):
-        log.debug("makeMove: move=%s self.pondermove=%s board1=%s board2=%s self.board=%s\n" % \
-            (move, self.pondermove, board1, board2, self.board), self.defname)
+        log.debug("makeMove: move=%s self.pondermove=%s board1=%s board2=%s self.board=%s" % \
+            (move, self.pondermove, board1, board2, self.board), extra={"task":self.defname})
         assert self.readyMoves
         
         with self.moveLock:
-            self.board = board1
+            self._recordMove(board1, move, board2)
             self.waitingForMove = True
             ponderhit = False
             
@@ -173,13 +228,15 @@ class UCIEngine (ProtocolEngine):
                 ponderhit = True
             elif board2 and self.pondermove:
                 self.ignoreNext = True
-                print >> self.engine, "stop"
+                print("stop", file=self.engine)
             
             self._searchNow(ponderhit=ponderhit)
         
         # Parse outputs
         try:
             r = self.returnQueue.get()
+            if r == "invalid":
+                raise InvalidMove
             if r == "del":
                 raise PlayerIsDead
             if r == "int":
@@ -210,98 +267,110 @@ class UCIEngine (ProtocolEngine):
     
     def setOptionAnalyzing (self, mode):
         self.mode = mode
+        if self.mode == INVERSE_ANALYZING:
+            self.board = self.gameBoard.switchColor()
     
     def setOptionInitialBoard (self, model):
-        log.debug("setOptionInitialBoard: self=%s, model=%s\n" % \
-            (self, model), self.defname)
-        # UCI always sets the position when searching for a new game, but for
-        # getting analyzers ready to analyze at first ply, it is good to have.
-        self.board = model.getBoardAtPly(model.ply)
-        pass
+        log.debug("setOptionInitialBoard: self=%s, model=%s" % \
+            (self, model), extra={"task":self.defname})
+        self._recordMoveList(model)
     
     def setOptionVariant (self, variant):
-        if variant == FischerRandomChess:
+        if variant == FischerandomBoard:
             assert self.hasOption("UCI_Chess960")
             self.setOption("UCI_Chess960", True)
+        elif self.hasOption("UCI_Variant") and not variant.standard_rules:
+            self.setOption("UCI_Variant", variant.cecp_name)
     
     def setOptionTime (self, secs, gain):
         self.wtime = int(max(secs*1000*self.timeHandicap, 1))
         self.btime = int(max(secs*1000*self.timeHandicap, 1))
         self.incr = int(gain*1000*self.timeHandicap)
     
-    def setOptionStrength (self, strength):
+    def setOptionStrength (self, strength, forcePonderOff):
         self.strength = strength
         
-        if self.hasOption('UCI_LimitStrength') and strength <= 6:
+        if self.hasOption('UCI_LimitStrength') and strength <= 18:
             self.setOption('UCI_LimitStrength', True)
             if self.hasOption('UCI_Elo'):
-                self.setOption('UCI_Elo', 300 * strength + 200)
+                self.setOption('UCI_Elo', 150 * strength)
         
-        if not self.hasOption('UCI_Elo') or strength == 7:
-            self.timeHandicap = th = 0.01 * 10**(strength/4.)
+        # Stockfish offers 20 skill levels
+        if self.hasOption('Skill Level') and strength <= 19:
+            self.setOption('Skill Level', strength)
+
+        if ((not self.hasOption('UCI_Elo')) and (not self.hasOption('Skill Level'))) or strength <= 19:
+            self.timeHandicap = th = 0.01 * 10**(strength/10.)
             self.wtime = int(max(self.wtime*th, 1))
             self.btime = int(max(self.btime*th, 1))
             self.incr = int(self.incr*th)
         
         if self.hasOption('Ponder'):
-            self.setOption('Ponder', strength >= 7)
-    
+            self.setOption('Ponder', strength >= 19 and not forcePonderOff)
+
+        if self.hasOption('GaviotaTbPath') and strength == 20:
+            self.setOption('GaviotaTbPath', conf.get("egtb_path", ""))
     #===========================================================================
     #    Interacting with the player
     #===========================================================================
     
     def pause (self):
-        log.debug("pause: self=%s\n" % self, self.defname)
+        log.debug("pause: self=%s" % self, extra={"task":self.defname})
         self.engine.pause()
         return
         
-        if self.board and self.board.color == self.color or \
+        if self.board.color == self.color or \
                 self.mode != NORMAL or self.pondermove:
             self.ignoreNext = True
-            print >> self.engine, "stop"
+            print("stop", file=self.engine)
     
     def resume (self):
-        log.debug("resume: self=%s\n" % self, self.defname)
+        log.debug("resume: self=%s" % self, extra={"task":self.defname})
         self.engine.resume()
         return
         
         if self.mode == NORMAL:
-            if self.board and self.board.color == self.color:
+            if self.board.color == self.color:
                 self._searchNow()
-            elif self.getOption('Ponder') and self.pondermove:
+            elif self.ponderOn and self.pondermove:
                 self._startPonder()
         else:
             self._searchNow()
     
     def hurry (self):
-        log.debug("hurry: self.waitingForMove=%s self.readyForStop=%s\n" % \
-            (self.waitingForMove, self.readyForStop), self.defname)
+        log.debug("hurry: self.waitingForMove=%s self.readyForStop=%s" % \
+            (self.waitingForMove, self.readyForStop), extra={"task":self.defname})
         # sending this more than once per move will crash most engines
         # so we need to send only the first one, and then ignore every "hurry" request
         # after that until there is another outstanding "position..go"
         with self.moveLock:
             if self.waitingForMove and self.readyForStop:
-                print >> self.engine, "stop"
+                print("stop", file=self.engine)
                 self.readyForStop = False
     
     def playerUndoMoves (self, moves, gamemodel):
-        log.debug("playerUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s\n" % \
-            (moves, gamemodel.ply, gamemodel.boards[-1], self.board), self.defname)
+        log.debug("playerUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s" % \
+            (moves, gamemodel.ply, gamemodel.boards[-1], self.board), extra={"task":self.defname})
+
+        self._recordMoveList(gamemodel)
         
         if (gamemodel.curplayer != self and moves % 2 == 1) or \
                 (gamemodel.curplayer == self and moves % 2 == 0):
             # Interrupt if we were searching but should no longer do so, or
             # if it is was our move before undo and it is still our move after undo
             # since we need to send the engine the new FEN in makeMove()
-            log.debug("playerUndoMoves: putting 'int' into self.returnQueue=%s\n" % \
-                self.returnQueue.queue, self.defname)
+            log.debug("playerUndoMoves: putting 'int' into self.returnQueue=%s" % \
+                self.returnQueue.queue, extra={"task":self.defname})
             self.returnQueue.put("int")
     
     def spectatorUndoMoves (self, moves, gamemodel):
-        log.debug("spectatorUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s\n" % \
-            (moves, gamemodel.ply, gamemodel.boards[-1], self.board), self.defname)
+        log.debug("spectatorUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s" % \
+            (moves, gamemodel.ply, gamemodel.boards[-1], self.board), extra={"task":self.defname})
+
+        self._recordMoveList(gamemodel)
         
-        self.putMove(gamemodel.getBoardAtPly(gamemodel.ply), None, None)
+        if self.readyMoves:
+            self._searchNow()
     
     #===========================================================================
     #    Offer handling
@@ -324,8 +393,9 @@ class UCIEngine (ProtocolEngine):
             engineDiscoverer or use the getOption, getOptions and hasOption
             methods, while you are in your 'readyForOptions' signal handler """ 
         if self.readyMoves:
-            log.warn("Options set after 'readyok' are not sent to the engine", self.defname)
+            log.warning("Options set after 'readyok' are not sent to the engine", extra={"task":self.defname})
         self.optionsToBeSent[key] = value
+        self.ponderOn = key=="Ponder" and value is True
     
     def getOption (self, option):
         assert self.readyOptions
@@ -346,11 +416,12 @@ class UCIEngine (ProtocolEngine):
     #===========================================================================
     
     def _newGame (self):
-        print >> self.engine, "ucinewgame"
+        print("ucinewgame", file=self.engine)
     
     def _searchNow (self, ponderhit=False):
-        log.debug("_searchNow: self.needBestmove=%s ponderhit=%s self.board=%s\n" % \
-            (self.needBestmove, ponderhit, self.board), self.defname)
+        log.debug("_searchNow: self.needBestmove=%s ponderhit=%s self.board=%s" % \
+            (self.needBestmove, ponderhit, self.board), extra={"task":self.defname})
+
         with self.moveLock:
             commands = []
             
@@ -358,63 +429,69 @@ class UCIEngine (ProtocolEngine):
                 commands.append("ponderhit")
                 
             elif self.mode == NORMAL:
-                commands.append("position fen %s" % self.board.asFen())
+                commands.append("position %s" % self.uciPosition)
                 if self.strength <= 3:
                     commands.append("go depth %d" % self.strength)
                 else:
-                    commands.append("go wtime %d btime %d winc %d binc %d" % \
-                                    (self.wtime, self.btime, self.incr, self.incr))
+                    commands.append("go wtime %d winc %d btime %d binc %d" % \
+                                    (self.wtime, self.incr, self.btime, self.incr))
                 
             else:
+                print("stop", file=self.engine)
+                
                 if self.mode == INVERSE_ANALYZING:
                     if self.board.board.opIsChecked():
                         # Many engines don't like positions able to take down enemy
                         # king. Therefore we just return the "kill king" move
                         # automaticaly
-                        self.emit("analyze", [getMoveKillingKing(self.board)], MATE_VALUE-1)
+                        self.emit("analyze", [([toAN(self.board, getMoveKillingKing(self.board))], MATE_VALUE-1, "")])
                         return
-                
-                print >> self.engine, "stop"
-                if self.board.asFen() == FEN_START:
-                    commands.append("position startpos")
+                    commands.append("position fen %s" % self.board.asFen())
                 else:
-                    commands.append("position fen %s" % self.board.asXFen())
-                commands.append("go infinite")
+                    commands.append("position %s" % self.uciPosition)
+
+                #commands.append("go infinite")
+                move_time = int(conf.get("max_analysis_spin", 3))*1000
+                commands.append("go movetime %s" % move_time)
+
+            if self.hasOption("MultiPV") and self.multipvSetting > 1:
+                self.multipvExpected = min(self.multipvSetting, legalMoveCount(self.board))
+            else:
+                self.multipvExpected = 1
+            self.analysis = [None] * self.multipvExpected
             
             if self.needBestmove:
                 self.commands.append(commands)
-                log.debug("_searchNow: self.needBestmove==True, appended to self.commands=%s\n" % \
-                    self.commands, self.defname)
+                log.debug("_searchNow: self.needBestmove==True, appended to self.commands=%s" % \
+                    self.commands, extra={"task":self.defname})
             else:
                 for command in commands:
-                    print >> self.engine, command
-                if self.board.asFen() != FEN_START and getStatus(self.board)[1] != WON_MATE:
+                    print(command, file=self.engine)
+                if getStatus(self.board)[1] != WON_MATE: # XXX This looks fishy.
                     self.needBestmove = True
                     self.readyForStop = True
     
     def _startPonder (self):
-        print >> self.engine, "position fen", self.board.asXFen(), \
-                                "moves", toAN(self.board, self.pondermove, short=True)
-        print >> self.engine, "go ponder wtime", self.wtime, \
-            "btime", self.btime, "winc", self.incr, "binc", self.incr
+        uciPos = self.uciPosition
+        if not self.uciPositionListsMoves:
+            uciPos += " moves"
+        print("position", uciPos, \
+                                self._moveToUCI(self.board, self.pondermove), file=self.engine)
+        print("go ponder wtime", self.wtime, \
+            "winc", self.incr, "btime", self.btime, "binc", self.incr, file=self.engine)
     
     #===========================================================================
     #    Parsing from engine
     #===========================================================================
     
-    def parseLines (self, engine, lines):
-        for line in lines:
-            self.__parseLine(line)
-    
-    def __parseLine (self, line):
+    def parseLine (self, engine, line):
         if not self.connected: return
         parts = line.split()
         if not parts: return
-        
         #---------------------------------------------------------- Initializing
         if parts[0] == "id":
-            self.ids[parts[1]] = " ".join(parts[2:])
             if parts[1] == "name":
+                self.ids[parts[1]] = " ".join(parts[2:])
                 self.setName(self.ids["name"])
             return
         
@@ -431,7 +508,7 @@ class UCIEngine (ProtocolEngine):
             dic = {}
             last = 1
             varlist = []
-            for i in xrange (2, len(parts)+1):
+            for i in range (2, len(parts)+1):
                 if i == len(parts) or parts[i] in OPTKEYS:
                     key = parts[last]
                     value = " ".join(parts[last+1:i])
@@ -440,16 +517,16 @@ class UCIEngine (ProtocolEngine):
                         
                     if key == "var":
                         varlist.append(value)
+                    elif key == "type" and value == "string":
+                        dic[key] = "text"
                     else:
                         dic[key] = value
                         
                     last = i
             if varlist:
-                dic["vars"] = varlist
+                dic["choices"] = varlist
             
-            name = dic["name"]
-            del dic["name"]
-            self.options[name] = dic
+            self.options[dic["name"]] = dic
             return
         
         #---------------------------------------------------------------- A Move
@@ -459,43 +536,50 @@ class UCIEngine (ProtocolEngine):
                 self.__sendQueuedGo()
                 
                 if self.ignoreNext:
-                    log.debug("__parseLine: line='%s' self.ignoreNext==True, returning\n" % \
-                        line.strip(), self.defname)
+                    log.debug("__parseLine: line='%s' self.ignoreNext==True, returning" % \
+                        line.strip(), extra={"task":self.defname})
                     self.ignoreNext = False
                     self.readyForStop = True
                     return
                 
+                movestr = parts[1]
                 if not self.waitingForMove:
-                    log.warn("__parseLine: self.waitingForMove==False, ignoring move=%s\n" % \
-                        parts[1], self.defname)
+                    log.warning("__parseLine: self.waitingForMove==False, ignoring move=%s" % \
+                        movestr, extra={"task":self.defname})
                     self.pondermove = None
                     return
                 self.waitingForMove = False
-                
-                move = parseAN(self.board, parts[1])
+
+                try:
+                    move = parseAny(self.board, movestr)
+                except ParsingError as e:
+                    self.invalid_move = movestr
+                    self.end(WHITEWON if self.board.color == BLACK else BLACKWON, WON_ADJUDICATION)
+                    return
                 
                 if not validate(self.board, move):
                     # This is critical. To avoid game stalls, we need to resign on
                     # behalf of the engine.
-                    log.error("__parseLine: move=%s didn't validate, putting 'del' in returnQueue. self.board=%s\n" % \
-                        (repr(move), self.board), self.defname)
-                    self.returnQueue.put('del')
+                    log.error("__parseLine: move=%s didn't validate, putting 'del' in returnQueue. self.board=%s" % \
+                        (repr(move), self.board), extra={"task":self.defname})
+                    self.invalid_move = movestr
+                    self.end(WHITEWON if self.board.color == BLACK else BLACKWON, WON_ADJUDICATION)
                     return
                 
-                self.board = self.board.move(move)
-                log.debug("__parseLine: applied move=%s to self.board=%s\n" % \
-                    (move, self.board), self.defname)
+                self._recordMove(self.board.move(move), move, self.board)
+                log.debug("__parseLine: applied move=%s to self.board=%s" % \
+                    (move, self.board), extra={"task":self.defname})
                 
-                if self.getOption('Ponder'):
+                if self.ponderOn:
                     self.pondermove = None
                     # An engine may send an empty ponder line, simply to clear.
-                    if len(parts) == 4 and self.board:
+                    if len(parts) == 4:
                         # Engines don't always check for everything in their
                         # ponders. Hence we need to validate.
                         # But in some cases, what they send may not even be
                         # correct AN - specially in the case of promotion.
                         try:
-                            pondermove = parseAN(self.board, parts[3])
+                            pondermove = parseAny(self.board, parts[3])
                         except ParsingError:
                             pass
                         else:
@@ -504,12 +588,15 @@ class UCIEngine (ProtocolEngine):
                                 self._startPonder()
                 
                 self.returnQueue.put(move)
-                log.debug("__parseLine: put move=%s into self.returnQueue=%s\n" % \
-                    (move, self.returnQueue.queue), self.defname)
+                log.debug("__parseLine: put move=%s into self.returnQueue=%s" % \
+                    (move, self.returnQueue.queue), extra={"task":self.defname})
                 return
         
         #----------------------------------------------------------- An Analysis
         if self.mode != NORMAL and parts[0] == "info" and "pv" in parts:
+            multipv = 1
+            if "multipv" in parts:
+                multipv = int(parts[parts.index("multipv")+1])
             scoretype = parts[parts.index("score")+1]
             if scoretype in ('lowerbound', 'upperbound'):
                 score = None
@@ -517,27 +604,28 @@ class UCIEngine (ProtocolEngine):
                 score = int(parts[parts.index("score")+2])
                 if scoretype == 'mate':
 #                    print >> self.engine, "stop"
-                    sign = score/abs(score)
-                    score = sign * (MATE_VALUE-abs(score))
+                    if score != 0:
+                        sign = score/abs(score)
+                        score = sign*MATE_VALUE
             
             movstrs = parts[parts.index("pv")+1:]
-            try:
-                moves = listToMoves (self.board, movstrs, AN, validate=True, ignoreErrors=False)
-            except ParsingError, e:
-                # ParsingErrors may happen when parsing "old" lines from
-                # analyzing engines, which haven't yet noticed their new tasks
-                log.debug("__parseLine: Ignored (%s) from analyzer: ParsingError%s\n" % \
-                    (' '.join(movstrs),e), self.defname)
-                return
-            
-            self.emit("analyze", moves, score)
+
+            if "depth" in parts:
+                depth = parts[parts.index("depth")+1]
+            else:
+                depth = ""
+                
+            if multipv <= len(self.analysis):
+                self.analysis[multipv - 1] = (movstrs, score, depth)
+
+            self.emit("analyze", self.analysis)
             return
         
         #-----------------------------------------------  An Analyzer bestmove
         if self.mode != NORMAL and parts[0] == "bestmove":
             with self.moveLock:
-                log.debug("__parseLine: processing analyzer bestmove='%s'\n" % \
-                    line.strip(), self.defname)
+                log.debug("__parseLine: processing analyzer bestmove='%s'" % \
+                    line.strip(), extra={"task":self.defname})
                 self.needBestmove = False
                 self.__sendQueuedGo(sendlast=True)
                 return
@@ -545,7 +633,7 @@ class UCIEngine (ProtocolEngine):
         #  Stockfish complaining it received a 'stop' without a corresponding 'position..go'
         if line.strip() == "Unknown command: stop":
             with self.moveLock:
-                log.debug("__parseLine: processing '%s'\n" % line.strip(), self.defname)
+                log.debug("__parseLine: processing '%s'" % line.strip(), extra={"task":self.defname})
                 self.ignoreNext = False
                 self.needBestmove = False
                 self.readyForStop = False
@@ -578,18 +666,34 @@ class UCIEngine (ProtocolEngine):
                     commands = self.commands.popleft()
                 
                 for command in commands:
-                    print >> self.engine, command
+                    print(command, file=self.engine)
                 self.needBestmove = True
                 self.readyForStop = True
-                log.debug("__sendQueuedGo: sent queued go=%s\n" % commands, self.defname)
+                log.debug("__sendQueuedGo: sent queued go=%s" % commands, extra={"task":self.defname})
 
     #===========================================================================
     #    Info
     #===========================================================================
     
-    def canAnalyze (self):
-        # All UCIEngines can analyze
-        return True
+    def maxAnalysisLines (self):
+        try:
+            return int(self.options["MultiPV"]["max"])
+        except (KeyError, ValueError):
+            return 1 # Engine does not support the MultiPV option
+        
+    def requestMultiPV (self, n):
+        multipvMax = self.maxAnalysisLines()
+        n = min(n, multipvMax)
+        
+        if n != self.multipvSetting:
+            conf.set("multipv", n)
+            with self.moveLock:
+                self.multipvSetting  = n
+                print("stop", file=self.engine)
+                print("setoption name MultiPV value", n, file=self.engine)
+                self._searchNow()
+        
+        return n
     
     def __repr__ (self):
         if self.name:
