@@ -1,18 +1,15 @@
 """ The task of this module is to provide easy saving/loading of configurations
     It also supports gconf like connection, so you get notices when a property
     has changed. """
+from __future__ import absolute_import
 
-# gconf's notify all seams to be broken
-#try:
-#    import gconf
-#    import conf_gconf as confmodule
-#except:
-import conf_configParser as confmodule
+from . import conf_configParser as confmodule
 
 """Module for using gconf without having to care about types"""
 
 def notify_add (key, func, *args):
     """The signature for func must be self, client, *args, **kwargs"""
+    assert isinstance(key, str)
     return confmodule.notify_add(key, func, args)
 
 def notify_remove (conid):
