@@ -1,35 +1,41 @@
-from pychess.System import conf, glock
+import colorsys
+import re
+import sys
+import webbrowser
+from threading import Thread
+
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GObject
+from gi.repository import Pango
+from gi.repository.GdkPixbuf import Pixbuf
+
+from pychess.compat import Queue, Empty,PY3
+from pychess.System import conf, fident
 from pychess.System.Log import log
-from pychess.System.ThreadPool import pool
 from pychess.System.prefix import addDataPrefix
 from pychess.widgets.ToggleComboBox import ToggleComboBox
-import Queue
-import colorsys
-import gtk.glade
-import pango
-import re
-import webbrowser
 
 
-def createCombo (combo, data):
-    ls = gtk.ListStore(gtk.gdk.Pixbuf, str)
-    for icon, label in data:
-        ls.append([icon, label])
+def createCombo (combo, data=[]):
+    ls = Gtk.ListStore(Pixbuf, str)
+    for row in data:
+        ls.append(row)
     combo.clear()
-    
+
     combo.set_model(ls)
-    crp = gtk.CellRendererPixbuf()
+    crp = Gtk.CellRendererPixbuf()
     crp.set_property('xalign',0)
     crp.set_property('xpad', 2)
     combo.pack_start(crp, False)
     combo.add_attribute(crp, 'pixbuf', 0)
-    
-    crt = gtk.CellRendererText()
+
+    crt = Gtk.CellRendererText()
     crt.set_property('xalign',0)
     crt.set_property('xpad', 4)
     combo.pack_start(crt, True)
     combo.add_attribute(crt, 'text', 1)
-    #crt.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)
+    #crt.set_property('ellipsize', Pango.EllipsizeMode.MIDDLE)
 
 
 def updateCombo (combo, data):
@@ -77,28 +83,38 @@ def genColor (n, startpoint=0):
 def keepDown (scrolledWindow):
     def changed (vadjust):
         if not hasattr(vadjust, "need_scroll") or vadjust.need_scroll:
-            vadjust.set_value(vadjust.upper-vadjust.page_size)
+            vadjust.set_value(vadjust.get_upper()-vadjust.get_page_size())
             vadjust.need_scroll = True
     scrolledWindow.get_vadjustment().connect("changed", changed)
-        
+
     def value_changed (vadjust):
-        vadjust.need_scroll = abs(vadjust.value + vadjust.page_size - \
-                vadjust.upper) < vadjust.step_increment
+        vadjust.need_scroll = abs(vadjust.get_value() + vadjust.get_page_size() - \
+                vadjust.get_upper()) < vadjust.get_step_increment()
     scrolledWindow.get_vadjustment().connect("value-changed", value_changed)
 
 
 
-def appendAutowrapColumn (treeview, defwidth, name, **kvargs):
-    cell = gtk.CellRendererText()
-    cell.props.wrap_mode = pango.WRAP_WORD
-    cell.props.wrap_width = defwidth
-    column = gtk.TreeViewColumn(name, cell, **kvargs)
+# wrap analysis text column. thanks to
+# http://www.islascruz.org/html/index.php?blog/show/Wrap-text-in-a-TreeView-column.html
+def appendAutowrapColumn (treeview, name, **kvargs):
+    cell = Gtk.CellRendererText()
+    #cell.props.wrap_mode = Pango.WrapMode.WORD
+    # TODO:
+    # changed to ellipsize instead until "never ending grow" bug gets fixed
+    # see https://github.com/pychess/pychess/issues/1054
+    cell.props.ellipsize = Pango.EllipsizeMode.END
+    column = Gtk.TreeViewColumn(name, cell, **kvargs)
     treeview.append_column(column)
-    
+
     def callback (treeview, allocation, column, cell):
-        otherColumns = (c for c in treeview.get_columns() if c != column)
+        otherColumns = [c for c in treeview.get_columns() if c != column]
         newWidth = allocation.width - sum(c.get_width() for c in otherColumns)
-        newWidth -= treeview.style_get_property("horizontal-separator") * 2
+
+        hsep = GObject.Value()
+        hsep.init(GObject.TYPE_INT)
+        hsep.set_int(0)
+        treeview.style_get_property("horizontal-separator", hsep)
+        newWidth -= hsep.get_int() * (len(otherColumns)+1) * 2
         if cell.props.wrap_width == newWidth or newWidth <= 0:
             return
         cell.props.wrap_width = newWidth
@@ -108,46 +124,49 @@ def appendAutowrapColumn (treeview, defwidth, name, **kvargs):
             store.row_changed(store.get_path(iter), iter)
             iter = store.iter_next(iter)
         treeview.set_size_request(0,-1)
-    treeview.connect_after("size-allocate", callback, column, cell)
-    
+    #treeview.connect_after("size-allocate", callback, column, cell)
+
     scroll = treeview.get_parent()
-    if isinstance(scroll, gtk.ScrolledWindow):
-        scroll.set_policy(gtk.POLICY_NEVER,
+    if isinstance(scroll, Gtk.ScrolledWindow):
+        scroll.set_policy(Gtk.PolicyType.NEVER,
                           scroll.get_policy()[1])
-    
+
     return cell
 
 
 METHODS = (
-    # gtk.SpinButton should be listed prior to gtk.Entry, as it is a
+    # Gtk.SpinButton should be listed prior to Gtk.Entry, as it is a
     # subclass, but requires different handling
-    (gtk.SpinButton, ("get_value", "set_value", "value-changed")),
-    (gtk.Entry, ("get_text", "set_text", "changed")),
-    (gtk.Expander, ("get_expanded", "set_expanded", "notify::expanded")),
-    (gtk.ComboBox, ("get_active", "set_active", "changed")),
-    # gtk.ToggleComboBox should be listed prior to gtk.ToggleButton, as it is a
+    (Gtk.SpinButton, ("get_value", "set_value", "value-changed")),
+    (Gtk.Entry, ("get_text", "set_text", "changed")),
+    (Gtk.Expander, ("get_expanded", "set_expanded", "notify::expanded")),
+    (Gtk.ComboBox, ("get_active", "set_active", "changed")),
+    # Gtk.ToggleComboBox should be listed prior to Gtk.ToggleButton, as it is a
     # subclass, but requires different handling
     (ToggleComboBox, ("_get_active", "_set_active", "changed")),
-    (gtk.ToggleButton, ("get_active", "set_active", "toggled")),
-    (gtk.CheckMenuItem, ("get_active", "set_active", "toggled")),
-    (gtk.Range, ("get_value", "set_value", "value-changed")))
+    (Gtk.IconView, ("_get_active", "_set_active", "selection-changed")),
+    (Gtk.ToggleButton, ("get_active", "set_active", "toggled")),
+    (Gtk.CheckMenuItem, ("get_active", "set_active", "toggled")),
+    (Gtk.Range, ("get_value", "set_value", "value-changed")),
+    (Gtk.TreeSortable, ("get_value", "set_value", "sort-column-changed")),
+    )
 
 def keep (widget, key, get_value_=None, set_value_=None, first_value=None):
     if widget == None:
-        raise AttributeError, "key '%s' isn't in widgets" % key
-    
+        raise AttributeError("key '%s' isn't in widgets" % key)
+
     for class_, methods_ in METHODS:
         if isinstance(widget, class_):
             getter, setter, signal = methods_
             break
     else:
-        raise AttributeError, "I don't have any knowledge of type: '%s'" % widget
-    
+        raise AttributeError("I don't have any knowledge of type: '%s'" % widget)
+
     if get_value_:
         get_value = lambda: get_value_(widget)
     else:
         get_value = getattr(widget, getter)
-    
+
     if set_value_:
         set_value = lambda v: set_value_(widget, v)
     else:
@@ -157,45 +176,45 @@ def keep (widget, key, get_value_=None, set_value_=None, first_value=None):
         try:
             v = conf.getStrict(key)
         except TypeError:
-            log.warn("uistuff.keep.setFromConf: Key '%s' from conf had the wrong type '%s', ignored" % \
+            log.warning("uistuff.keep.setFromConf: Key '%s' from conf had the wrong type '%s', ignored" % \
                      (key, type(conf.getStrict(key))))
             if first_value != None:
                 conf.set(key, first_value)
             else: conf.set(key, get_value())
         else:
             set_value(v)
-    
+
     def callback(*args):
         if not conf.hasKey(key) or conf.getStrict(key) != get_value():
             conf.set(key, get_value())
     widget.connect(signal, callback)
     conf.notify_add(key, lambda *args: setFromConf())
-    
+
     if conf.hasKey(key):
         setFromConf()
     elif first_value != None:
         conf.set(key, first_value)
 
 # loadDialogWidget() and saveDialogWidget() are similar to uistuff.keep() but are needed
-# for saving widget values for gtk.Dialog instances that are loaded with different
+# for saving widget values for Gtk.Dialog instances that are loaded with different
 # sets of values/configurations and which also aren't instant save like in
 # uistuff.keep(), but rather are saved later if and when the user clicks
 # the dialog's OK button
 def loadDialogWidget (widget, widget_name, config_number, get_value_=None,
                       set_value_=None, first_value=None):
     key = widget_name + "-" + str(config_number)
-    
+
     if widget == None:
-        raise AttributeError, "key '%s' isn't in widgets" % widget_name
-    
+        raise AttributeError("key '%s' isn't in widgets" % widget_name)
+
     for class_, methods_ in METHODS:
         if isinstance(widget, class_):
             getter, setter, signal = methods_
             break
     else:
         if set_value_ == None:
-            raise AttributeError, "I don't have any knowledge of type: '%s'" % widget
-    
+            raise AttributeError("I don't have any knowledge of type: '%s'" % widget)
+
     if get_value_:
         get_value = lambda: get_value_(widget)
     else:
@@ -205,12 +224,12 @@ def loadDialogWidget (widget, widget_name, config_number, get_value_=None,
         set_value = lambda v: set_value_(widget, v)
     else:
         set_value = getattr(widget, setter)
-    
+
     if conf.hasKey(key):
         try:
             v = conf.getStrict(key)
         except TypeError:
-            log.warn("uistuff.loadDialogWidget: Key '%s' from conf had the wrong type '%s', ignored" % \
+            log.warning("uistuff.loadDialogWidget: Key '%s' from conf had the wrong type '%s', ignored" % \
                      (key, type(conf.getStrict(key))))
             if first_value != None:
                 conf.set(key, first_value)
@@ -221,23 +240,23 @@ def loadDialogWidget (widget, widget_name, config_number, get_value_=None,
         conf.set(key, first_value)
         set_value(conf.getStrict(key))
     else:
-        log.warn("Didn't load widget \"%s\": no conf value and no first_value arg" % \
+        log.warning("Didn't load widget \"%s\": no conf value and no first_value arg" % \
                  widget_name)
 
 def saveDialogWidget (widget, widget_name, config_number, get_value_=None):
     key = widget_name + "-" + str(config_number)
-    
+
     if widget == None:
-        raise AttributeError, "key '%s' isn't in widgets" % widget_name
-    
+        raise AttributeError("key '%s' isn't in widgets" % widget_name)
+
     for class_, methods_ in METHODS:
         if isinstance(widget, class_):
             getter, setter, signal = methods_
             break
     else:
         if get_value_ == None:
-            raise AttributeError, "I don't have any knowledge of type: '%s'" % widget
-    
+            raise AttributeError("I don't have any knowledge of type: '%s'" % widget)
+
     if get_value_:
         get_value = lambda: get_value_(widget)
     else:
@@ -249,42 +268,62 @@ def saveDialogWidget (widget, widget_name, config_number, get_value_=None):
 POSITION_NONE, POSITION_CENTER, POSITION_GOLDEN = range(3)
 def keepWindowSize (key, window, defaultSize=None, defaultPosition=POSITION_NONE):
     """ You should call keepWindowSize before show on your windows """
-    
+
     key = key + "window"
-    
+
     def savePosition (window, *event):
-        
+        log.debug("keepWindowSize.savePosition: %s" % window.get_title())
         width = window.get_allocation().width
         height = window.get_allocation().height
         x, y = window.get_position()
-        
+
         if width <= 0:
             log.error("Setting width = '%d' for %s to conf" % (width,key))
         if height <= 0:
             log.error("Setting height = '%d' for %s to conf" % (height,key))
-        
+
+        log.debug("Saving window position width=%s height=%s x=%s y=%s" %
+                  (width, height, x, y))
         conf.set(key+"_width",  width)
         conf.set(key+"_height", height)
         conf.set(key+"_x", x)
         conf.set(key+"_y", y)
+
+        return False
     window.connect("delete-event", savePosition, "delete-event")
-    
+
     def loadPosition (window):
+        #log.debug("keepWindowSize.loadPosition: %s" % window.title)
         width, height = window.get_size_request()
-        
+
         if conf.hasKey(key+"_width") and conf.hasKey(key+"_height"):
             width = conf.getStrict(key+"_width")
             height = conf.getStrict(key+"_height")
+            log.debug("Resizing window to width=%s height=%s" % (width, height))
             window.resize(width, height)
-        
+
         elif defaultSize:
             width, height = defaultSize
+            log.debug("Resizing window to width=%s height=%s" % (width, height))
             window.resize(width, height)
-        
+
+        elif key=="mainwindow":
+            monitor_x, monitor_y, monitor_width, monitor_height = getMonitorBounds()
+            width = int(monitor_width/2)
+            height = int(monitor_height/4)*3
+            log.debug("Resizing window to width=%s height=%s" % (width, height))
+            window.resize(width, height)
+
+        elif key=="preferencesdialogwindow":
+            window.resize(1, 1)
+
+
         if conf.hasKey(key+"_x") and conf.hasKey(key+"_y"):
+            log.debug("Moving window to x=%s y=%s" % (conf.getStrict(key+"_x"),
+                                                      conf.getStrict(key+"_y")))
             window.move(conf.getStrict(key+"_x"),
                         conf.getStrict(key+"_y"))
-        
+
         elif defaultPosition in (POSITION_CENTER, POSITION_GOLDEN):
             monitor_x, monitor_y, monitor_width, monitor_height = getMonitorBounds()
             x = int(monitor_width/2-width/2) + monitor_x
@@ -293,10 +332,11 @@ def keepWindowSize (key, window, defaultSize=None, defaultPosition=POSITION_NONE
             else:
                 # Place the window on the upper golden ratio line
                 y = int(monitor_height/2.618-height/2) + monitor_y
+            log.debug("Moving window to x=%s y=%s" % (x, y))
             window.move(x, y)
-        
+
     loadPosition(window)
-    
+
     # In rare cases, gtk throws some gtk_size_allocation error, which is
     # probably a race condition. To avoid the window forgets its size in
     # these cases, we add this extra hook
@@ -313,28 +353,29 @@ def onceWhenReady(window, func, *args, **kwargs):
     handler_id = window.connect_after("size-allocate", cb, func, *args, **kwargs)
 
 def getMonitorBounds():
-    screen = gtk.gdk.screen_get_default()
+    screen = Gdk.Screen.get_default()
     root_window = screen.get_root_window()
-    mouse_x, mouse_y, mouse_mods = root_window.get_pointer()
+    ptr_window, mouse_x, mouse_y, mouse_mods = root_window.get_pointer()
     current_monitor_number = screen.get_monitor_at_point(mouse_x,mouse_y)
     monitor_geometry = screen.get_monitor_geometry(current_monitor_number)
     return monitor_geometry.x, monitor_geometry.y, monitor_geometry.width, monitor_geometry.height
 
 
-tooltip = gtk.Window(gtk.WINDOW_POPUP)
+tooltip = Gtk.Window(Gtk.WindowType.POPUP)
 tooltip.set_name('gtk-tooltip')
 tooltip.ensure_style()
 tooltipStyle = tooltip.get_style()
 
 def makeYellow (box):
-    def on_box_expose_event (box, event):
-        box.style.paint_flat_box (box.window,
-            gtk.STATE_NORMAL, gtk.SHADOW_NONE, None, box, "tooltip",
-            box.allocation.x, box.allocation.y,
-            box.allocation.width, box.allocation.height)
+    def on_box_expose_event (box, context):
+        #box.style.paint_flat_box (box.window,
+        #    Gtk.StateType.NORMAL, Gtk.ShadowType.NONE, None, box, "tooltip",
+        #    box.allocation.x, box.allocation.y,
+        #    box.allocation.width, box.allocation.height)
+        pass
     def cb (box):
         box.set_style(tooltipStyle)
-        box.connect("expose-event", on_box_expose_event)
+        box.connect("draw", on_box_expose_event)
     onceWhenReady(box, cb)
 
 
@@ -345,14 +386,14 @@ emailre = re.compile("[\w\.]+@[\w\.]+\.\w{2,4}")
 def initTexviewLinks (textview, text):
     tags = []
     textbuffer = textview.get_buffer()
-    
+
     while True:
         linkmatch = linkre.search(text)
         emailmatch = emailre.search(text)
         if not linkmatch and not emailmatch:
             textbuffer.insert (textbuffer.get_end_iter(), text)
             break
-        
+
         if emailmatch and (not linkmatch or \
                 emailmatch.start() < linkmatch.start()):
             s = emailmatch.start()
@@ -365,18 +406,18 @@ def initTexviewLinks (textview, text):
                 e -= 1
             type = "link"
         textbuffer.insert (textbuffer.get_end_iter(), text[:s])
-        
+
         tag = textbuffer.create_tag (None, foreground="blue",
-                underline=pango.UNDERLINE_SINGLE)
+                underline=Pango.Underline.SINGLE)
         tags.append([tag, text[s:e], type, textbuffer.get_end_iter()])
-        
+
         textbuffer.insert_with_tags (
                 textbuffer.get_end_iter(), text[s:e], tag)
-        
+
         tags[-1].append(textbuffer.get_end_iter())
-        
+
         text = text[e:]
-    
+
     def on_press_in_textview (textview, event):
         iter = textview.get_iter_at_location (int(event.x), int(event.y))
         if not iter: return
@@ -384,97 +425,67 @@ def initTexviewLinks (textview, text):
             if iter.has_tag(tag):
                 tag.props.foreground = "red"
                 break
-    
+
     def on_release_in_textview (textview, event):
         iter = textview.get_iter_at_location (int(event.x), int(event.y))
         if not iter: return
         for tag, link, type, s, e in tags:
             if iter and iter.has_tag(tag) and \
-                    tag.props.foreground_gdk.red == 0xffff:
+                    tag.props.foreground_Gdk.red == 0xffff:
                 if type == "link":
                     webbrowser.open(link)
                 else: webbrowser.open("mailto:"+link)
             tag.props.foreground = "blue"
-    
-    stcursor = gtk.gdk.Cursor(gtk.gdk.XTERM)
-    linkcursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
+
+    stcursor = Gdk.Cursor(Gdk.CursorType.XTERM)
+    linkcursor = Gdk.Cursor(Gdk.CursorType.HAND2)
     def on_motion_in_textview(textview, event):
-        textview.window.get_pointer()
+        #textview.get_window().get_pointer()
         iter = textview.get_iter_at_location (int(event.x), int(event.y))
         if not iter: return
         for tag, link, type, s, e in tags:
             if iter.has_tag(tag):
-                textview.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor (
+                textview.get_window(Gtk.TextWindowType.TEXT).set_cursor (
                         linkcursor)
                 break
-        else: textview.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(stcursor)
-    
+        else: textview.get_window(Gtk.TextWindowType.TEXT).set_cursor(stcursor)
+
     textview.connect ("motion-notify-event", on_motion_in_textview)
     textview.connect ("leave_notify_event", on_motion_in_textview)
     textview.connect("button_press_event", on_press_in_textview)
     textview.connect("button_release_event", on_release_in_textview)
 
 
-
-def LinkLabel (text, url):
-    label = gtk.Label()
-    
-    eventbox = gtk.EventBox()
-    label.set_markup("<span color='blue'><u>%s</u></span>" % text)
-    eventbox.add(label)
-    
-    def released (eventbox, event):
-        webbrowser.open(url)
-        label.set_markup("<span color='blue'><u>%s</u></span>" % text)
-    eventbox.connect("button_release_event", released)
-    
-    def pressed (eventbox, event):
-        label.set_markup("<span color='red'><u>%s</u></span>" % text)
-    eventbox.connect("button_press_event", pressed)
-    
-    eventbox.connect_after("realize",
-        lambda w: w.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2)))
-    
-    return eventbox
-
-cachedGlades = {}
-def cacheGladefile(filename):
-    """ gtk.glade automatically caches the file, so we only need to use this
-        file once """
-    if filename not in cachedGlades:
-        cachedGlades[filename] = Queue.Queue()
-        def readit ():
-            glade = gtk.glade.XML(addDataPrefix("glade/%s" % filename))
-            cachedGlades[filename].put(glade)
-        pool.start(readit)
+no_gettext = False
 
 class GladeWidgets:
     """ A simple class that wraps a the glade get_widget function
         into the python __getitem__ version """
     def __init__ (self, filename):
-        self.glade = None
-        try:
-            if filename in cachedGlades:
-                self.glade = cachedGlades[filename].get(block=False)
-        except Queue.Empty:
-            pass
-        
-        if not self.glade:
-            glock.acquire()
-#            print "uistuff.py:gladefile = %s" % filename
-            self.glade = gtk.glade.XML(addDataPrefix("glade/%s" % filename))
-            glock.release()
-        
-        self.extras = {}
-    
+        self.builder = Gtk.Builder()
+        if not no_gettext:
+            self.builder.set_translation_domain("pychess")
+        self.builder.add_from_file(addDataPrefix("glade/%s" % filename))
+
+        # TODO: remove this when upstream fixes translations with Python3+Windows
+        if PY3 and sys.platform == "win32" and not no_gettext:
+            for obj in self.builder.get_objects():
+                if (not isinstance(obj, Gtk.SeparatorMenuItem)) and hasattr(obj, "get_label"):
+                    label = obj.get_label()
+                    if label is not None:
+                        obj.set_label(_(label))
+                elif hasattr(obj, "get_title"):
+                    title = obj.get_title()
+                    if title is not None:
+                        obj.set_title(_(title))
+                if hasattr(obj, "get_tooltip_text"):
+                    text = obj.get_tooltip_text()
+                    if text is not None:
+                        obj.set_tooltip_text(_(text))
+
     def __getitem__(self, key):
-        if key in self.extras:
-            return self.extras[key]
-        return self.glade.get_widget(key)
-    
-    def __setitem__(self, key, widget):
-        self.extras[key] = widget
-    
+        return self.builder.get_object(key)
+
     def getGlade (self):
-        return self.glade
+        return self.builder
 
