@@ -1,7 +1,10 @@
+from __future__ import absolute_import
+from __future__ import print_function
+
 from pychess.Utils.Cord import Cord
 from pychess.Utils.const import *
-from lutils import lmove
-from lutils.lmove import ParsingError, FLAG_PIECE
+from pychess.Utils.lutils.lmovegen import newMove
+from .lutils import lmove
 
 class Move:
     
@@ -14,10 +17,7 @@ class Move:
         if not cord1:
             self.move = cord0
             self.flag = self.move >> 12
-            if self.flag in PROMOTIONS:
-                self.promotion = lmove.PROMOTE_PIECE (self.move)
-            else: self.promotion = QUEEN
-            self.cord0 = Cord(lmove.FCORD(self.move))
+            self.cord0 = None if self.flag == DROP else Cord(lmove.FCORD(self.move))
             self.cord1 = Cord(lmove.TCORD(self.move))
             
         else:
@@ -26,16 +26,23 @@ class Move:
             self.cord0 = cord0
             self.cord1 = cord1
             if not board:
-                raise ValueError, "Move needs a Board object in order to investigate flags"
+                raise ValueError("Move needs a Board object in order to investigate flags")
             
             self.flag = NORMAL_MOVE
-            
-            if board[self.cord0].piece == PAWN and  self.cord1.y in (0,7):
-                if promotion == None: promotion = QUEEN
-                self.flag = FLAG_PIECE(promotion)
+            if board[self.cord0].piece == PAWN and  self.cord1.cord in board.PROMOTION_ZONE[board.board.color]:
+                if promotion == None:
+                    if board.variant == SITTUYINCHESS:
+                        if cord0 == cord1:
+                            self.flag = lmove.FLAG_PIECE(QUEEN)
+                    else:
+                        self.flag = lmove.FLAG_PIECE(QUEEN)
+                else:
+                    self.flag = lmove.FLAG_PIECE(promotion)
             
             elif board[self.cord0].piece == KING:
-                if board.variant == FISCHERRANDOMCHESS:
+                if self.cord0 == self.cord1:
+                    self.flag = NULL_MOVE
+                elif board.variant == FISCHERRANDOMCHESS:
                     if (abs(self.cord0.x - self.cord1.x) > 1 and self.cord1.x==C1) or \
                         \
                         (board.board.ini_rooks[board.color][0] == self.cord1.cord and \
@@ -48,11 +55,11 @@ class Move:
                         ((board.board.color == WHITE and board.board.castling & W_OO) or \
                         (board.board.color == BLACK and board.board.castling & B_OO))):
                         self.flag = KING_CASTLE
-                else:
+                elif board.variant != CAMBODIANCHESS:
                     if self.cord0.x - self.cord1.x == 2:
-                        self.flag = QUEEN_CASTLE
+                        self.flag = QUEEN_CASTLE if self.cord0.x == 4 else KING_CASTLE
                     elif self.cord0.x - self.cord1.x == -2:
-                        self.flag = KING_CASTLE
+                        self.flag = KING_CASTLE if self.cord0.x == 4 else QUEEN_CASTLE
             
             elif board[self.cord0].piece == PAWN and \
                     board[self.cord1] == None and \
@@ -60,7 +67,7 @@ class Move:
                     self.cord0.y != self.cord1.y:
                 self.flag = ENPASSANT
             
-            self.move = lmove.newMove(self.cord0.cord, self.cord1.cord, self.flag)
+            self.move = newMove(self.cord0.cord, self.cord1.cord, self.flag)
             
     def _get_cords (self):
         return (self.cord0, self.cord1)
@@ -73,7 +80,14 @@ class Move:
     promotion = property(_get_promotion)
     
     def __repr__ (self):
-        return str(self.cord0) + str(self.cord1)
+        promotion = "="+reprSign[lmove.PROMOTE_PIECE(self.flag)] \
+                    if self.flag in PROMOTIONS else ""
+        
+        if self.flag == DROP:
+            piece = reprSign[lmove.FCORD(self.move)]
+            return piece + "@" +  str(self.cord1) + promotion
+        else:
+            return str(self.cord0) + str(self.cord1) + promotion
 
     def __eq__ (self, other):
         if isinstance(other, Move):
@@ -125,11 +139,11 @@ def parseAN (board, an):
 def listToSan (board, moves):
     return lmove.listToSan(board.board, (m.move for m in moves))
 
-def toAN (board, move, short=False):
+def toAN (board, move, short=False, castleNotation=CASTLE_SAN):
     """ Returns a Algebraic Notation string of a move
         board should be prior to the move """
     
-    return lmove.toAN (board.board, move.move, short=short)
+    return lmove.toAN (board.board, move.move, short=short, castleNotation=castleNotation)
     
 def toSAN (board, move, localRepr=False):
     """ Returns a Short/Abbreviated Algebraic Notation string of a move 
