@@ -1,28 +1,37 @@
 import heapq
 from time import time
-from gobject import SIGNAL_RUN_FIRST, TYPE_NONE, GObject
+
+from gi.repository import GLib, GObject
+
 from pychess.Utils.const import WHITE, BLACK
 from pychess.System import repeat
+from pychess.System.Log import log
 
-class TimeModel (GObject):
+class TimeModel (GObject.GObject):
     
     __gsignals__ = {
-        "player_changed": (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
-        "time_changed": (SIGNAL_RUN_FIRST, TYPE_NONE, ()),
-        "zero_reached": (SIGNAL_RUN_FIRST, TYPE_NONE, (int,)),
-        "pause_changed": (SIGNAL_RUN_FIRST, TYPE_NONE, (bool,))
+        "player_changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "time_changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "zero_reached": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        "pause_changed": (GObject.SignalFlags.RUN_FIRST, None, (bool,))
     }
+
     
     ############################################################################
     # Initing                                                                  #
     ############################################################################
     
-    def __init__ (self, secs, gain, bsecs=-1):
-        GObject.__init__(self)
-        
+    def __init__ (self, secs=0, gain=0, bsecs=-1, minutes=-1):        
+        GObject.GObject.__init__(self)
         if bsecs < 0: bsecs = secs
+        if minutes < 0:
+            minutes = secs / 60
+        self.minutes = minutes  # The number of minutes for the original starting
+            # time control (not necessarily where the game was resumed,
+            # i.e. self.intervals[0][0])
         self.intervals = [[secs],[bsecs]]
         self.gain = gain
+        self.secs = secs
         
         self.paused = False
         # The left number of secconds at the time pause was turned on
@@ -37,44 +46,53 @@ class TimeModel (GObject):
         self.connect('time_changed', self.__zerolistener, 'time_changed')
         self.connect('player_changed', self.__zerolistener, 'player_changed')
         self.connect('pause_changed', self.__zerolistener, 'pause_changed')
-        self.heap = []
+
+        self.zero_listener_id = None
+        self.zero_listener_time = 0
+        self.zero_listener_source = None
+        
+    def __repr__ (self):
+        s = "<TimeModel object at %s (White: %s Black: %s ended=%s)>" % \
+            (id(self), str(self.getPlayerTime(WHITE)),
+             str(self.getPlayerTime(BLACK)), self.ended)
+        return s
     
+    @property
+    def ply(self):
+        return len(self.intervals[0]) + len(self.intervals[1]) - 2
+        
     def __zerolistener(self, *args):
-        # If we are called by a sleeper (rather than a signal) we need to pop
-        # at least one time, as we might otherwise end up with items in the
-        # heap, but no sleepers.
-        if len(args) == 0 and self.heap:
-            self.heap.pop()
-        # Pop others (could this give a problem where too many are popped?)
-        # No I don't think so. If a sleeper is too slow, so a later sleeper
-        # comes before him and pops him, then it is most secure not to rely on
-        # mr late, and start a new one.
-        # We won't be 'one behind' always, because the previous pop doesnt
-        # happen if the heap is empty.
-        while self.heap and self.heap[-1] <= time():
-            self.heap.pop()
+        if self.ended:
+            return False
         
-        if self.getPlayerTime(WHITE) <= 0:
-            #print 'emit for white'
-            self.emit('zero_reached', WHITE)
-        if self.getPlayerTime(BLACK) <= 0:
-            #print 'emit for black'
-            self.emit('zero_reached', BLACK)
+        cur_time = time()
+        t1 = cur_time + self.getPlayerTime(WHITE)
+        t2 = cur_time + self.getPlayerTime(BLACK)
+        if t1 <= t2:
+            t = t1
+            color = WHITE
+        else:
+            t = t2
+            color = BLACK
         
-        #print 'heap is now', self.heap
-        
-        t1 = time() + self.getPlayerTime(WHITE)
-        t2 = time() + self.getPlayerTime(BLACK)
-        t = min(t1,t2)
-        
-        if not self.heap or t < self.heap[-1]:
-            s = t-time()+0.01
-            if s > 0:
-                self.heap.append(t)
-                # Because of recur, we wont get callback more than once.
-                repeat.repeat_sleep(self.__zerolistener, s, recur=True)
-                #print 'repeat on', s
+        s = t-cur_time+0.01
+        if s > 0 and s != self.zero_listener_time:
+            if (self.zero_listener_id is not None) and \
+                (self.zero_listener_source is not None) and \
+                not self.zero_listener_source.is_destroyed():
+                GLib.source_remove(self.zero_listener_id)
+            self.zero_listener_time = s
+            self.zero_listener_id = GLib.timeout_add(10, self.__checkzero, color)
+            default_context = GLib.main_context_get_thread_default() or GLib.main_context_default()
+            if hasattr(default_context, "find_source_by_id"):
+                self.zero_listener_source = default_context.find_source_by_id(self.zero_listener_id)
     
+    def __checkzero(self, color):
+        if self.getPlayerTime(color) <= 0:
+            self.emit('zero_reached', color)
+            return False
+        return True
+        
     ############################################################################
     # Interacting                                                              #
     ############################################################################
@@ -84,7 +102,6 @@ class TimeModel (GObject):
         self.emit("player_changed")
     
     def tap (self):
-        
         if self.paused: return
         
         if self.started:
@@ -95,8 +112,8 @@ class TimeModel (GObject):
         else:
             self.intervals[self.movingColor].append (
                     self.intervals[self.movingColor][-1] )
-            
-            if len(self.intervals[0]) + len(self.intervals[1]) >= 4:
+            # FICS rule
+            if self.ply >= 2:
                 self.started = True
         
         self.movingColor = 1-self.movingColor
@@ -106,19 +123,24 @@ class TimeModel (GObject):
             self.emit("time_changed")
         
         self.emit("player_changed")
-    
+
     def start (self):
         if self.started: return
         self.started = True
-        
         self.counter = time()
         self.emit("time_changed")
     
     def end (self):
+        log.debug("TimeModel.end: self=%s" % self)
         self.pause()
         self.ended = True
+        if (self.zero_listener_id is not None) and \
+            (self.zero_listener_source is not None) and \
+            not self.zero_listener_source.is_destroyed():
+            GLib.source_remove(self.zero_listener_id)
     
     def pause (self):
+        log.debug("TimeModel.pause: self=%s" % self)
         if self.paused: return
         self.paused = True
         
@@ -130,6 +152,7 @@ class TimeModel (GObject):
         self.emit("pause_changed", True)
     
     def resume (self):
+        log.debug("TimeModel.resume: self=%s" % self)
         if not self.paused: return
         
         self.paused = False
@@ -155,7 +178,7 @@ class TimeModel (GObject):
         if not self.started:
             self.start()
         
-        for i in xrange(moves):
+        for i in range(moves):
             self.movingColor = 1-self.movingColor
             del self.intervals[self.movingColor][-1]
         
@@ -175,34 +198,46 @@ class TimeModel (GObject):
     def updatePlayer (self, color, secs):
         if color == self.movingColor and self.started:
             self.counter = secs + time() - self.intervals[color][-1]
-        else: self.intervals[color][-1] = secs
-        self.emit("time_changed")
-    
-    def syncClock (self, wsecs, bsecs):
-        """ Syncronize clock to e.g. fics time """
-        if self.movingColor == WHITE:
-            if self.started:
-                self.counter = wsecs + time() - self.intervals[WHITE][-1]
-            else: self.intervals[WHITE][-1] = wsecs
-            self.intervals[BLACK][-1] = bsecs
-        else:
-            if self.started:
-                self.counter = bsecs + time() - self.intervals[BLACK][-1]
-            else: self.intervals[BLACK][-1] = bsecs
-            self.intervals[WHITE][-1] = wsecs
+        self.intervals[color][-1] = secs
         self.emit("time_changed")
     
     ############################################################################
     # Info                                                                     #
     ############################################################################
     
-    def getPlayerTime (self, color):
-        if color == self.movingColor and self.started:
+    def getPlayerTime (self, color, movecount=-1):
+        if color == self.movingColor and self.started and movecount == -1:
             if self.paused:
-                return self.intervals[color][-1] - self.pauseInterval
+                return self.intervals[color][movecount] - self.pauseInterval
             elif self.counter:
-                return self.intervals[color][-1] - (time() - self.counter)
-        return self.intervals[color][-1]
+                return self.intervals[color][movecount] - (time() - self.counter)
+        return self.intervals[color][movecount]
     
     def getInitialTime (self):
         return self.intervals[WHITE][0]
+
+    def getElapsedMoveTime (self, ply):
+        movecount, color = divmod(ply+1, 2)
+        gain = self.gain if ply > 2 else 0
+        if len(self.intervals[color]) > movecount:
+            return self.intervals[color][movecount-1] - self.intervals[color][movecount] + gain if movecount > 1 else 0
+        else:
+            return 0
+        
+    @property
+    def display_text (self):
+        t = ("%d " % self.minutes) + _("min")
+        if self.gain != 0:
+            t += (" + %d " % self.gain) + _("sec")
+        return t
+
+    @property
+    def hasTimes(self):
+        return len(self.intervals[0]) > 1
+        
+    @property
+    def ply(self):
+        return len(self.intervals[BLACK]) + len(self.intervals[WHITE]) - 2
+
+    def hasBWTimes(self, bmovecount, wmovecount):
+        return len(self.intervals[BLACK]) > bmovecount and len(self.intervals[WHITE]) > wmovecount
