@@ -1,11 +1,11 @@
 # -*- coding: UTF-8 -*-
+from __future__ import unicode_literals
 
 ################################################################################
 # PyChess information                                                          #
 ################################################################################
 
 NAME = "PyChess"
-ENGINES_XML_API_VERSION = "0.10.1"
 
 ################################################################################
 # Player info                                                                  #
@@ -27,7 +27,7 @@ WHITE, BLACK = range(2)
 # Game states
 WAITING_TO_START, PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON, KILLED, \
     ADJOURNED, ABORTED, UNKNOWN_STATE = range(10)
-reprResult = ["*", "*", "*", "1/2-1/2", "1-0", "0-1", "?", "*", "?", "?"]
+reprResult = ["*", "*", "*", "1/2-1/2", "1-0", "0-1", "*", "*", "*", "*"]
 
 UNDOABLE_STATES = (DRAW, WHITEWON, BLACKWON)
 UNFINISHED_STATES = (WAITING_TO_START, PAUSED, RUNNING, UNKNOWN_STATE)
@@ -35,11 +35,20 @@ UNFINISHED_STATES = (WAITING_TO_START, PAUSED, RUNNING, UNKNOWN_STATE)
 # Chess variants
 NORMALCHESS, CORNERCHESS, SHUFFLECHESS, FISCHERRANDOMCHESS, RANDOMCHESS, \
 ASYMMETRICRANDOMCHESS, UPSIDEDOWNCHESS, PAWNSPUSHEDCHESS, PAWNSPASSEDCHESS, \
-LOSERSCHESS, PAWNODDSCHESS, KNIGHTODDSCHESS, ROOKODDSCHESS, QUEENODDSCHESS, \
-BLINDFOLDCHESS, HIDDENPAWNSCHESS, HIDDENPIECESCHESS, ALLWHITECHESS = range(18)
+THEBANCHESS, PAWNODDSCHESS, KNIGHTODDSCHESS, ROOKODDSCHESS, QUEENODDSCHESS, \
+BLINDFOLDCHESS, HIDDENPAWNSCHESS, HIDDENPIECESCHESS, ALLWHITECHESS, \
+ATOMICCHESS, BUGHOUSECHESS, CRAZYHOUSECHESS, LOSERSCHESS, SUICIDECHESS, \
+WILDCASTLECHESS, WILDCASTLESHUFFLECHESS, KINGOFTHEHILLCHESS, THREECHECKCHESS, \
+ASEANCHESS, MAKRUKCHESS, SITTUYINCHESS, CAMBODIANCHESS, AIWOKCHESS, \
+EUROSHOGICHESS, SETUPCHESS = range(34)
+
+ASEAN_VARIANTS = (ASEANCHESS, MAKRUKCHESS, CAMBODIANCHESS, AIWOKCHESS, SITTUYINCHESS)
+DROP_VARIANTS = (BUGHOUSECHESS, CRAZYHOUSECHESS, EUROSHOGICHESS, SITTUYINCHESS, SETUPCHESS)
+UNSUPPORTED = (BUGHOUSECHESS, AIWOKCHESS, EUROSHOGICHESS, SETUPCHESS)
 
 # Chess variant groups
-VARIANTS_BLINDFOLD, VARIANTS_ODDS, VARIANTS_SHUFFLE, VARIANTS_OTHER = range(4)
+VARIANTS_BLINDFOLD, VARIANTS_ODDS, VARIANTS_SHUFFLE, VARIANTS_OTHER, \
+VARIANTS_OTHER_NONSTANDARD, VARIANTS_ASEAN = range(6)
 
 # Action errors
 ACTION_ERROR_NOT_OUT_OF_TIME, \
@@ -50,16 +59,17 @@ ACTION_ERROR_NOT_OUT_OF_TIME, \
 
 # Game state reasons
 ABORTED_ADJUDICATION, ABORTED_AGREEMENT, ABORTED_COURTESY, ABORTED_EARLY, \
-    ABORTED_SERVER_SHUTDOWN, ADJOURNED_COURTESY, \
+    ABORTED_SERVER_SHUTDOWN, ADJOURNED_COURTESY, ABORTED_DISCONNECTION, \
 ADJOURNED_AGREEMENT, ADJOURNED_LOST_CONNECTION, ADJOURNED_SERVER_SHUTDOWN, \
     ADJOURNED_COURTESY_WHITE, ADJOURNED_COURTESY_BLACK, \
     ADJOURNED_LOST_CONNECTION_WHITE, ADJOURNED_LOST_CONNECTION_BLACK, \
 DRAW_50MOVES, DRAW_ADJUDICATION, DRAW_AGREE, DRAW_CALLFLAG, DRAW_INSUFFICIENT, \
-    DRAW_LENGTH, DRAW_REPITITION, DRAW_STALEMATE, \
+    DRAW_EQUALMATERIAL, DRAW_LENGTH, DRAW_REPITITION, DRAW_STALEMATE, \
     DRAW_BLACKINSUFFICIENTANDWHITETIME, DRAW_WHITEINSUFFICIENTANDBLACKTIME, \
 WON_ADJUDICATION, WON_CALLFLAG, WON_DISCONNECTION, WON_MATE, WON_RESIGN, \
-    WON_NOMATERIAL, \
-WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, DISCONNECTED, UNKNOWN_REASON = range(33)
+    WON_LESSMATERIAL, WON_NOMATERIAL, WON_KINGEXPLODE, WON_KINGINCENTER, \
+    WON_THREECHECK, \
+WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, DISCONNECTED, UNKNOWN_REASON = range(39)
 
 UNDOABLE_REASONS = (DRAW_50MOVES, DRAW_INSUFFICIENT, DRAW_LENGTH,
                     DRAW_REPITITION, DRAW_STALEMATE, DRAW_AGREE, DRAW_CALLFLAG, \
@@ -110,35 +120,45 @@ NORMAL, ANALYZING, INVERSE_ANALYZING = range(3)
 ################################################################################
 
 # BPAWN is a pawn that moves in the opposite direction
-EMPTY, PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, BPAWN = range(8)
+EMPTY, PAWN, KNIGHT, BISHOP, ROOK, QUEEN, KING, BPAWN, \
+ASEAN_WBISHOP, ASEAN_BBISHOP, ASEAN_QUEEN = range(11)
 
 # Is sliding piece
-sliders = [ False, False, False, True, True, True, False, False ]
+sliders = [ False, False, False, True, True, True, False, False, \
+            False, False, False ]
 
 # Piece signs
 reprSign = ["", "P", "N", "B", "R", "Q", "K"]
-chr2Sign = {"k":KING, "q": QUEEN, "r": ROOK, "b": BISHOP, "n": KNIGHT, "p":PAWN}
+reprSignMakruk = ["", "P", "N", "S", "R", "M", "K"]
+reprSignSittuyin = ["", "P", "N", "S", "R", "F", "K"]
+chr2Sign = {"k":KING, "q": QUEEN, "r": ROOK, "b": BISHOP, "n": KNIGHT, "p":PAWN,
+            "m": QUEEN, "s": BISHOP, "f": QUEEN}
+chrU2Sign = {"K":KING, "Q": QUEEN, "R": ROOK, "B": BISHOP, "N": KNIGHT, "P":PAWN,
+            "M": QUEEN, "S": BISHOP, "F": QUEEN}
 
 ################################################################################
 # Move values                                                                  #
 ################################################################################
 
 NORMAL_MOVE, QUEEN_CASTLE, KING_CASTLE, ENPASSANT, \
-KNIGHT_PROMOTION, BISHOP_PROMOTION, ROOK_PROMOTION, QUEEN_PROMOTION = range(8)
+KNIGHT_PROMOTION, BISHOP_PROMOTION, ROOK_PROMOTION, QUEEN_PROMOTION, KING_PROMOTION, NULL_MOVE, DROP = range(11)
+PROMOTIONS = (KING_PROMOTION, QUEEN_PROMOTION, ROOK_PROMOTION, BISHOP_PROMOTION, KNIGHT_PROMOTION)
 
-PROMOTIONS = (QUEEN_PROMOTION, ROOK_PROMOTION, BISHOP_PROMOTION, KNIGHT_PROMOTION)
 # Algebraic notation types: Short, Long, Figure and Simpe
 SAN, LAN, FAN, AN = range(4)
+# Castling notation types: e.g., O-O, e1g1, e1h1
+CASTLE_SAN, CASTLE_KK, CASTLE_KR = range(3)
 
 FAN_PIECES = [
-    ["", u"♙", u"♘", u"♗", u"♖", u"♕", u"♔", ""],
-    ["", u"♟", u"♞", u"♝", u"♜", u"♛", u"♚", ""]
+    ["", "♙", "♘", "♗", "♖", "♕", "♔", ""],
+    ["", "♟", "♞", "♝", "♜", "♛", "♚", ""]
 ]
 
 ################################################################################
 # Castling values                                                              #
 ################################################################################
 W_OO, W_OOO, B_OO, B_OOO = [2**i for i in range(4)]
+CAS_FLAGS = ((W_OOO,W_OO),(B_OOO,B_OO))
 W_CASTLED, B_CASTLED = [2**i for i in range(2)]
 
 ################################################################################
@@ -172,29 +192,12 @@ cordDic = {}
 for cord, name in enumerate(reprCord):
     cordDic[name] = cord
 
-################################################################################
-# Internet Chess                                                               #
-################################################################################
-
-IC_CONNECTED, IC_DISCONNECTED = range(2)
-
-IC_POS_ISOLATED, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, \
-IC_POS_OP_TO_MOVE, IC_POS_ME_TO_MOVE, IC_POS_OBSERVING, IC_POS_INITIAL = range(7)
-
-# Rating deviations
-DEVIATION_NONE, DEVIATION_ESTIMATED, DEVIATION_PROVISIONAL = range(3)
-
-IC_STATUS_PLAYING, IC_STATUS_ACTIVE, IC_STATUS_BUSY, IC_STATUS_OFFLINE, \
-IC_STATUS_AVAILABLE, IC_STATUS_NOT_AVAILABLE, IC_STATUS_EXAMINING, \
-IC_STATUS_IDLE, IC_STATUS_IN_TOURNAMENT, IC_STATUS_RUNNING_SIMUL_MATCH, \
-IC_STATUS_UNKNOWN = range(11)
-
 ################################################################################
 # User interface                                                               #
 ################################################################################
 
 # Hint modes
-HINT, SPY = ["hint", "spy"]
+OPENING, ENDGAME, HINT, SPY = ["opening", "endgame", "hint", "spy"]
 
 # Sound settings
 SOUND_MUTE, SOUND_BEEP, SOUND_SELECT, SOUND_URI = range(4)
@@ -203,23 +206,16 @@ SOUND_MUTE, SOUND_BEEP, SOUND_SELECT, SOUND_URI = range(4)
 CLEAR, ENPAS = range(2)
 
 # Main menu items
-GAME_MENU_ITEMS = ("save_game1", "save_game_as1", "properties1", "close1")
+GAME_MENU_ITEMS = ("save_game1", "save_game_as1", "share_game", "export_position1",
+                    "analyze_game1", "properties1", "close1")
 ACTION_MENU_ITEMS = ("abort", "adjourn", "draw", "pause1", "resume1", "undo1", 
                      "call_flag", "resign", "ask_to_move")
 VIEW_MENU_ITEMS = ("rotate_board1", "show_sidepanels", "hint_mode", "spy_mode")
-MENU_ITEMS = GAME_MENU_ITEMS + ACTION_MENU_ITEMS + VIEW_MENU_ITEMS
-
-# Main menu items
-GAME_MENU_ITEMS = ("save_game1", "save_game_as1", "properties1", "close1")
-ACTION_MENU_ITEMS = ("abort", "adjourn", "draw", "pause1", "resume1", "undo1", 
-                     "call_flag", "resign", "ask_to_move")
-VIEW_MENU_ITEMS = ("rotate_board1", "show_sidepanels", "hint_mode", "spy_mode")
-MENU_ITEMS = GAME_MENU_ITEMS + ACTION_MENU_ITEMS + VIEW_MENU_ITEMS
+EDIT_MENU_ITEMS = ("copy_pgn", "copy_fen",)
+MENU_ITEMS = GAME_MENU_ITEMS + ACTION_MENU_ITEMS + VIEW_MENU_ITEMS + EDIT_MENU_ITEMS
 
 ################################################################################
 # Subprocess                                                                   #
 ################################################################################
 
 SUBPROCESS_PTY, SUBPROCESS_SUBPROCESS, SUBPROCESS_FORK = range(3)
-
-LOG_DEBUG, LOG_LOG, LOG_WARNING, LOG_ERROR = range(4)
