@@ -1,28 +1,30 @@
+from __future__ import absolute_import
 from time import time
 from random import random
 from heapq import heappush, heappop
 
-from lmovegen import genAllMoves, genCheckEvasions, genCaptures
+from .lmovegen import genAllMoves, genCheckEvasions, genCaptures
+from .egtb_gaviota import egtb_gaviota
 from pychess.Utils.const import *
-from leval import evaluateComplete
-from lsort import getCaptureValue, getMoveValue
-from lmove import toSAN
-from ldata import MATE_VALUE
-from TranspositionTable import TranspositionTable
-import ldraw
-from pychess.Utils.lutils.egtb_k4it import probeEndGameTable
+from .leval import evaluateComplete
+from .lsort import getCaptureValue, getMoveValue
+from .lmove import toSAN
+from .ldata import MATE_VALUE, VALUE_AT_PLY
+from .TranspositionTable import TranspositionTable
+from pychess.Variants.atomic import kingExplode
+from pychess.Variants.kingofthehill import testKingInCenter
+from pychess.Variants.threecheck import checkCount
+from . import ldraw
 
 TIMECHECK_FREQ = 500
 
-table = TranspositionTable(5000000)
+table = TranspositionTable(32 * 1024 * 1024)
 skipPruneChance = 0
 searching = False
-movesearches = 0
 nodes = 0
-last = 0
 endtime = 0
 timecheck_counter = TIMECHECK_FREQ
-useegtb = False
+egtb = None
 
 def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
     """ This is a alphabeta/negamax/quiescent/iterativedeepend search algorithm
@@ -38,31 +40,55 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
             the deepest)
         *   a score of your standing the the last possition. """
     
-    global last, searching, nodes, movesearches, table, endtime, timecheck_counter
+    global searching, nodes, table, endtime, timecheck_counter
     foundPv = False
     hashf = hashfALPHA
     amove = []
     
+    ############################################################################
+    # Mate distance pruning
+    ############################################################################
+
+    MATED = -MATE_VALUE+ply 
+    MATE_IN_1 = MATE_VALUE-ply-1
+
+    if beta <= MATED:
+        return [], MATED
+    if beta >= MATE_IN_1:
+        beta = MATE_IN_1
+        if alpha >= beta:
+            return [], MATE_IN_1    
+
+    if board.variant == ATOMICCHESS:
+        if bin(board.boards[board.color][KING]).count("1") == 0:
+            return [], MATED
+    elif board.variant == KINGOFTHEHILLCHESS:
+        if testKingInCenter(board):
+            return [], MATED
+    elif board.variant == THREECHECKCHESS:
+        if checkCount(board) == 3:
+            return [], MATED
+
     ############################################################################
     # Look in the end game table
     ############################################################################
     
-    if useegtb:
-        egtb = probeEndGameTable(board)
-        if egtb:
-            move, state, steps = egtb[0]
+    global egtb
+    if egtb:
+        tbhits = egtb.scoreAllMoves(board)
+        if tbhits:
+            move, state, steps = tbhits[0]
             
             if state == DRAW:
                 score = 0
             elif board.color == WHITE:
                 if state == WHITEWON:
-                    score = MATE_VALUE-steps+2
-                else: score = -MATE_VALUE+steps-2
+                    score = MATE_VALUE-steps
+                else: score = -MATE_VALUE+steps
             else:
                 if state == WHITEWON:
-                    score = -MATE_VALUE+steps-2
-                else: score = MATE_VALUE-steps+2
-            last = 1
+                    score = -MATE_VALUE+steps
+                else: score = MATE_VALUE-steps
             return [move], score
     
     ###########################################################################
@@ -73,32 +99,34 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
     # We don't adjudicate draws. Clients may have different rules for that.
     if ply > 0:
         if ldraw.test(board):
-            last = 2
             return [], 0
     
     ############################################################################
     # Look up transposition table                                              #
     ############################################################################
-    
-    table.setHashMove (depth, -1)
-    probe = table.probe (board, depth, alpha, beta)
-    hashmove = None
-    if probe:
-        move, score, hashf = probe
-        hashmove = move
-        table.setHashMove (depth, move)
-        
-        if hashf == hashfEXACT:
-            last = 3
-            return [move], score
-        elif hashf == hashfBETA:
-            beta = min(score, beta)
-        elif hashf == hashfALPHA:
-            alpha = score
+    # TODO: add holder to hash
+    if board.variant not in DROP_VARIANTS:
+        if ply == 0:
+            table.newSearch()
+
+        table.setHashMove (depth, -1)
+        probe = table.probe (board, depth, alpha, beta)
+        hashmove = None
+        if probe:
+            move, score, hashf = probe
+            score = VALUE_AT_PLY(score, ply)
+            hashmove = move
+            table.setHashMove (depth, move)
             
-        if hashf != hashfBAD and alpha >= beta:
-            last = 4
-            return [move], score
+            if hashf == hashfEXACT:
+                return [move], score
+            elif hashf == hashfBETA:
+                beta = min(score, beta)
+            elif hashf == hashfALPHA:
+                alpha = score
+                
+            if hashf != hashfBAD and alpha >= beta:
+                return [move], score
     
     ############################################################################
     # Cheking the time                                                         #
@@ -115,7 +143,6 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
     ############################################################################
     
     if not searching:
-        last = 5
         return [], -evaluateComplete(board, 1-board.color)
     
     ############################################################################
@@ -128,8 +155,9 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
         if isCheck:
             # Being in check is that serious, that we want to take a deeper look
             depth += 1
+        elif board.variant in (LOSERSCHESS, SUICIDECHESS, ATOMICCHESS):
+            return [], evaluateComplete(board, board.color)
         else:
-            last = 6
             mvs, val = quiescent(board, alpha, beta, ply)
             return mvs, val
     
@@ -137,11 +165,26 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
     # Find and sort moves                                                      #
     ############################################################################
     
-    movesearches += 1
-    
-    if isCheck:
-        moves = [(-getMoveValue(board,table,depth,m),m) for m in genCheckEvasions(board)]
-    else: moves = [(-getMoveValue(board,table,depth,m),m) for m in genAllMoves(board)]
+    if board.variant in (LOSERSCHESS, SUICIDECHESS):
+        mlist = [m for m in genCaptures(board)]
+        if board.variant == LOSERSCHESS and isCheck:
+            evasions = [m for m in genCheckEvasions(board)]
+            eva_cap = [m for m in evasions if m in mlist]
+            mlist = eva_cap if eva_cap else evasions
+        if not mlist and not isCheck:
+            mlist = [m for m in genAllMoves(board)]
+        moves = [(-getMoveValue(board,table,depth,m),m) for m in mlist]
+    elif board.variant == ATOMICCHESS:
+        if isCheck:
+            mlist = [m for m in genCheckEvasions(board) if not kingExplode(board, m, board.color)]
+        else:
+            mlist = [m for m in genAllMoves(board) if not kingExplode(board, m, board.color)]
+        moves = [(-getMoveValue(board,table,depth,m),m) for m in mlist]
+    else:
+        if isCheck:
+            moves = [(-getMoveValue(board,table,depth,m),m) for m in genCheckEvasions(board)]
+        else:
+            moves = [(-getMoveValue(board,table,depth,m),m) for m in genAllMoves(board)]
     moves.sort()
     
     # This is needed on checkmate
@@ -178,15 +221,14 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
         
         if val > alpha:
             if val >= beta:
-                if searching:
-                    table.record (board, move, beta, hashfBETA, depth)
+                if searching and move>>12 != DROP:
+                    table.record (board, move, VALUE_AT_PLY(beta, -ply), hashfBETA, depth)
                     # We don't want to use our valuable killer move spaces for
                     # captures and promotions, as these are searched early anyways.
                     if board.arBoard[move&63] == EMPTY and \
                             not move>>12 in PROMOTIONS:
                         table.addKiller (depth, move)
                         table.addButterfly(move, depth)
-                last = 7
                 return [move]+mvs, beta
             
             alpha = val
@@ -200,30 +242,26 @@ def alphaBeta (board, depth, alpha=-MATE_VALUE, beta=MATE_VALUE, ply=0):
     
     if amove:
         if searching:
-            table.record (board, amove[0], alpha, hashf, depth)
+            table.record (board, amove[0], VALUE_AT_PLY(alpha, -ply), hashf, depth)
             if board.arBoard[amove[0]&63] == EMPTY:
                 table.addKiller (depth, amove[0])
-        last = 8
         return amove, alpha
     
     if catchFailLow:
         if searching:
-            table.record (board, catchFailLow, alpha, hashf, depth)
-        last = 9
+            table.record (board, catchFailLow, VALUE_AT_PLY(alpha, -ply), hashf, depth)
         return [catchFailLow], alpha
 
     # If no moves were found, this must be a mate or stalemate
     if isCheck:
-        last = 10
-        return [], -MATE_VALUE+ply-2
+        return [], MATED
     
-    last = 11
     return [], 0
 
 def quiescent (board, alpha, beta, ply):
     
     if skipPruneChance and random() < skipPruneChance:
-        return [], (alpha+beta)/2
+        return [], (alpha+beta) // 2
     
     global nodes
     
@@ -232,12 +270,13 @@ def quiescent (board, alpha, beta, ply):
     
     isCheck = board.isChecked()
     
-    # Our quiescent search will evaluate the current board, and if 
-    value = evaluateComplete(board, board.color)
-    if value >= beta and not isCheck:
-        return [], beta
-    if value > alpha:
-        alpha = value
+    # no stand-pat when in check
+    if not isCheck: 
+        value = evaluateComplete(board, board.color)
+        if value >= beta:
+            return [], beta
+        if value > alpha:
+            alpha = value
     
     amove = []
     
@@ -250,7 +289,7 @@ def quiescent (board, alpha, beta, ply):
             # Heap.append is fine, as we don't really do sorting on the few moves
             heap.append((0, move))
         if not someMove:
-            return [], -MATE_VALUE+ply-2
+            return [], -MATE_VALUE+ply
     else:
         for move in genCaptures (board):
             heappush(heap, (-getCaptureValue (board, move), move))
@@ -262,9 +301,10 @@ def quiescent (board, alpha, beta, ply):
         v, move = heappop(heap)
         
         board.applyMove(move)
-        if board.opIsChecked():
-            board.popMove()
-            continue
+        if not isCheck:
+            if board.opIsChecked():
+                board.popMove()
+                continue
         
         mvs, val = quiescent(board, -beta, -alpha, ply+1)
         val = -val
@@ -283,3 +323,30 @@ def quiescent (board, alpha, beta, ply):
     
     else:
         return [], alpha
+
+
+class EndgameTable():
+    def __init__ (self):
+        self.provider = egtb_gaviota()
+    
+    def _pieceCounts (self, board):
+        return sorted([ bin(board.friends[i]).count("1") for i in range(2) ])
+    
+    def scoreAllMoves (self, lBoard):
+        """ Return each move's result and depth to mate.
+            
+            lBoard: A low-level board structure
+            Return value: a list, with best moves first, of:
+            move: A high-level move structure
+            game_result: Either WHITEWON, DRAW, BLACKWON
+            depth: Depth to mate
+        """
+        
+        pc = self._pieceCounts(lBoard)
+        if self.provider.supports(pc):
+            return self.provider.scoreAllMoves(lBoard)
+        return []
+
+def enableEGTB():
+    global egtb
+    egtb = EndgameTable()
