@@ -4,18 +4,17 @@
     scores, but strings describing the differences.
     Can be used for commenting on board changes.
 """
+from __future__ import absolute_import
 
-from gettext import ngettext
-from ldata import *
+from .ldata import *
 from pychess.Utils.const import *
 from pychess.Utils.lutils.attack import staticExchangeEvaluate, getAttacks, \
     defends
-from pychess.Utils.lutils.lmove import TCORD, FCORD, FLAG, PROMOTE_PIECE, toSAN, \
-    newMove
-from pychess.Utils.lutils.lmovegen import genCaptures, genAllMoves
+from pychess.Utils.lutils.lmove import TCORD, FCORD, FLAG, PROMOTE_PIECE, toSAN
+from pychess.Utils.lutils.lmovegen import genCaptures, genAllMoves, newMove
 from pychess.Utils.lutils.validator import validateMove
 from pychess.Utils.repr import reprColor, reprPiece
-import leval
+from . import leval
 
 
 def join(items):
@@ -104,7 +103,7 @@ def offencive_moves_rook (model, ply, phase):
     ffile = fileBits[FILE(FCORD(move))]
     tfile = fileBits[FILE(tcord)]
 
-    if ffile & pawns and not tfile & pawns and bitLength(pawns) >= 3:
+    if ffile & pawns and not tfile & pawns and bin(pawns).count("1") >= 3:
         if not tfile & oppawns:
             yield _("moves a rook to an open file")
         else: yield _("moves an rook to a half-open file")
@@ -165,6 +164,10 @@ def attack_type (model, ply, phase):
                     yield _("exchanges material")
                 elif see > 0:
                     yield _("captures material")
+    else:
+        see = staticExchangeEvaluate(oldboard, move)
+        if see < 0:
+            yield _("sacrifies material")
 
     PIECE_VALUES[BISHOP] = bishopBackup
 
@@ -305,7 +308,7 @@ def offencive_moves_pin (model, ply, phase):
     tcord = TCORD(move)
     piece = board.arBoard[tcord]
 
-    ray = createBoard(0)
+    ray = 0
     if piece in (BISHOP, QUEEN):
         ray |= (ray45[tcord] | ray135[tcord]) & ~(ray45[fcord] | ray135[fcord])
     if piece in (ROOK, QUEEN):
@@ -322,7 +325,7 @@ def offencive_moves_pin (model, ply, phase):
                 continue
             # There should be exactly one opponent piece in between
             op = clearBit(ray & board.friends[board.color], c)
-            if bitLength(op) != 1:
+            if bin(op).count("1") != 1:
                 continue
             # The king can't be pinned
             pinned = lastBit(op)
@@ -421,10 +424,10 @@ def state_pawn (model, ply, phase):
     for file in range(8):
         bits = fileBits[file]
 
-        count = bitLength(pawns & bits)
-        oldcount = bitLength(oldpawns & bits)
-        opcount = bitLength(oppawns & bits)
-        oldopcount = bitLength(oldoppawns & bits)
+        count = bin(pawns & bits).count("1")
+        oldcount = bin(oldpawns & bits).count("1")
+        opcount = bin(oppawns & bits).count("1")
+        oldopcount = bin(oldoppawns & bits).count("1")
 
         # Single pawn -> double pawns
         if count > oldcount >= 1:
@@ -540,8 +543,8 @@ def state_trappedBishops (model, ply, phase):
     else:
         return
 
-    s = leval.evalTrappedBishops (board, opcolor, phase)
-    olds = leval.evalTrappedBishops (oldboard, opcolor, phase)
+    s = leval.evalTrappedBishops (board, opcolor)
+    olds = leval.evalTrappedBishops (oldboard, opcolor)
 
     # We have got more points -> We have trapped a bishop
     if s > olds:
@@ -625,10 +628,10 @@ def tip_pawnStorm (model, ply, phase):
 
     wking = board.boards[WHITE][KING]
     bking = board.boards[BLACK][KING]
-    wleft = bitLength(board.boards[WHITE][PAWN] & left)
-    wright = bitLength(board.boards[WHITE][PAWN] & right)
-    bleft = bitLength(board.boards[BLACK][PAWN] & left)
-    bright = bitLength(board.boards[BLACK][PAWN] & right)
+    wleft = bin(board.boards[WHITE][PAWN] & left).count("1")
+    wright = bin(board.boards[WHITE][PAWN] & right).count("1")
+    bleft = bin(board.boards[BLACK][PAWN] & left).count("1")
+    bright = bin(board.boards[BLACK][PAWN] & right).count("1")
 
     if wking & left and bking & right:
         if wright > bright:
@@ -664,13 +667,6 @@ def tip_mobility (model, ply, phase):
 
     board.setColor(colorBackup)
 
-    #print wmoves, bmoves, phase
-
-    #wb = board.boards[WHITE]
-    #print float(wmoves)/bitLength(wb[KNIGHT]|wb[BISHOP]|wb[ROOK]|wb[QUEEN])
-    #bb = board.boards[WHITE]
-    #print float(bmoves)/bitLength(bb[KNIGHT]|bb[BISHOP]|bb[ROOK]|bb[QUEEN])
-
     if wmoves-phase >= (bmoves+1)*7:
         yield wmoves-bmoves, _("Black has a rather cramped position")
     elif wmoves-phase >= (bmoves+1)*3:
