@@ -1,3 +1,4 @@
+from __future__ import print_function
 # Random Chess
 
 import random
@@ -7,12 +8,23 @@ from pychess.Utils.Board import Board
 
 class RandomBoard(Board):
     variant = RANDOMCHESS
+    __desc__ = _("FICS wild/3: http://www.freechess.org/Help/HelpFiles/wild.html\n" +
+                 "* Randomly chosen pieces (two queens or three rooks possible)\n" +
+                 "* Exactly one king of each color\n" +
+                 "* Pieces placed randomly behind the pawns\n" +
+                 "* No castling\n" +
+                 "* Black's arrangement mirrors white's")
+    name = _("Random")
+    cecp_name = "unknown"
+    need_initial_board = True
+    standard_rules = True
+    variant_group = VARIANTS_SHUFFLE
 
-    def __init__ (self, setup=False):
+    def __init__ (self, setup=False, lboard=None):
         if setup is True:
-            Board.__init__(self, setup=self.random_start())
+            Board.__init__(self, setup=self.random_start(), lboard=lboard)
         else:
-            Board.__init__(self, setup=setup)
+            Board.__init__(self, setup=setup, lboard=lboard)
 
     def random_start(self):        
         tmp = random.sample(('r', 'n', 'b', 'q')*16, 7)
@@ -24,22 +36,7 @@ class RandomBoard(Board):
         return tmp
 
 
-class RandomChess:
-    __desc__ = _("FICS wild/3: http://www.freechess.org/Help/HelpFiles/wild.html\n" +
-                 "* Randomly chosen pieces (two queens or three rooks possible)\n" +
-                 "* Exactly one king of each color\n" +
-                 "* Pieces placed randomly behind the pawns\n" +
-                 "* No castling\n" +
-                 "* Black's arrangement mirrors white's")
-    name = _("Random")
-    cecp_name = "unknown"
-    board = RandomBoard
-    need_initial_board = True
-    standard_rules = True
-    variant_group = VARIANTS_SHUFFLE
-
-
 if __name__ == '__main__':
     Board = RandomBoard(True)
     for i in range(10):
-        print Board.random_start()
+        print(Board.random_start())
