@@ -1,18 +1,17 @@
+from gi.repository import GObject
 
-from gobject import *
+sanmove = "([a-hx@OoPKQRBN0-8+#=-]{2,7})"
 
-sanmove = "([a-hxOoKQRBN0-8+#=-]{2,7})"
-
-class ErrorManager (GObject):
+class ErrorManager (GObject.GObject):
     
     __gsignals__ = {
-        'onCommandNotFound' : (SIGNAL_RUN_FIRST, TYPE_NONE, (str,)),
-        'onAmbiguousMove' : (SIGNAL_RUN_FIRST, TYPE_NONE, (str,)),
-        'onIllegalMove' : (SIGNAL_RUN_FIRST, TYPE_NONE, (str,)),
+        'onCommandNotFound' : (GObject.SignalFlags.RUN_FIRST, None, (str,)),
+        'onAmbiguousMove' : (GObject.SignalFlags.RUN_FIRST, None, (str,)),
+        'onIllegalMove' : (GObject.SignalFlags.RUN_FIRST, None, (str,)),
     }
     
     def __init__ (self, connection):
-        GObject.__init__(self)
+        GObject.GObject.__init__(self)
         connection.expect_line (self.onError, "(.*?): Command not found\.")
         connection.expect_line (self.onAmbiguousMove, "Ambiguous move \((%s)\)\." % sanmove)
         connection.expect_line (self.onIllegalMove, "Illegal move \((%s)\)\." % sanmove)
